@@ -12,13 +12,12 @@
 #include "cli_interactive.h"
 
 //-----------------------------------------------------------------------------
-const KCmdLineOptions CLI::NullCmdLineOption = KCmdLineLastOption;
 const CLI::CommandData CLI::NullCommandData = { NULL, NoCommandProperty, 0, 0, NULL };
 const CLI::PropertyData CLI::NullPropertyData = { 0, 0, 0, 0, 0, 0 };
 
-namespace CLI
-{
-const KCmdLineOptions STANDARD_OPTIONS[] = {
+namespace CLI {
+
+const Piklab::CmdLineOption STANDARD_OPTIONS[] = {
   { "f",              0, 0 },
   { "force",          I18N_NOOP("Overwrite files and answer \"yes\" to questions."), 0 },
   { "i",              0, 0 },
@@ -33,7 +32,7 @@ const KCmdLineOptions STANDARD_OPTIONS[] = {
   { "quiet",          I18N_NOOP("Do not display messages."), 0 },
   { "log <file>",     I18N_NOOP("Specify log file."), 0 },
   { "replay <file>",  I18N_NOOP("Replay commands in provided file."), 0 },
-  NullCmdLineOption
+  { NULL, NULL, NULL }
 };
 
 const CLI::CommandData STANDARD_COMMAND_DATA[] = {
@@ -77,7 +76,7 @@ CLI::MainBase::~MainBase()
 void CLI::MainBase::initArgs(int argc, char** argv, const char* fileDescription)
 {
   static Piklab::OptionList list = optionList(fileDescription); // danger!
-  Piklab::initArgs(_aboutData, argc, argv, *list.ptr());
+  Piklab::initArgs(_aboutData, argc, argv, list);
   _args = KCmdLineArgs::parsedArgs();
 
   QString filepath = _args->getOption("log");
@@ -113,10 +112,10 @@ Piklab::OptionList CLI::MainBase::optionList(const char *fileDescription) const
 {
   Piklab::OptionList list;
   OptionIterator iter;
-  while (const KCmdLineOptions* data = iter.getNext())
+  while (const Piklab::CmdLineOption* data = iter.getNext())
     list.append(*data);
 
-  KCmdLineOptions opt;
+  Piklab::CmdLineOption opt;
   for (uint i=0; propertyData(i).name != NULL; i++) {
     opt.description = NULL;
     opt.def = NULL;
@@ -416,11 +415,7 @@ CLI::ExitCode CLI::MainBase::runInteractive()
   std::auto_ptr<Interactive> p(new Interactive(*this));
   log(Log::LineType::Normal, i18n("Interactive mode: type 'help' for the list of commands."));
   log(Log::LineType::Normal, QString::null);
-#if QT_VERSION<0x040000
-  return ExitCode(qApp->exec());
-#else
   return ExitCode(QCoreApplication::exec());
-#endif
 }
 
 CLI::ExitCode CLI::MainBase::runReplay(const QString &filepath)
@@ -576,10 +571,10 @@ CLI::MainBase::OptionIterator::OptionIterator()
 {
 }
 
-const KCmdLineOptions* CLI::MainBase::OptionIterator::getNext()
+const Piklab::CmdLineOption* CLI::MainBase::OptionIterator::getNext()
 {
   if (_standard) {
-    const KCmdLineOptions* data = &STANDARD_OPTIONS[_index];
+    const Piklab::CmdLineOption* data = &STANDARD_OPTIONS[_index];
     if (data->name == NULL) {
       _standard = false;
       _index = 0;
@@ -588,8 +583,8 @@ const KCmdLineOptions* CLI::MainBase::OptionIterator::getNext()
     ++_index;
     return data;
   } else {
-    const KCmdLineOptions* data = &MainBase::instance().option(_index);
-    if (data->name == NULL) return NULL;
+    const Piklab::CmdLineOption* data = MainBase::instance().option(_index);
+    if (data == NULL || data->name == NULL) return NULL;
     ++_index;
     return data;
   }
