@@ -11,33 +11,24 @@
 
 #include "common/common/global.h"
 
-class KProcess;
-
 namespace Process
 {
+class Base;
+
 class ProcessIO : public QObject
 {
 Q_OBJECT
 public:
   ProcessIO(QObject *parent = 0);
-  virtual ~ProcessIO();
   void setArguments(const QStringList &args);
-  void setWorkingDirectory(const QDir &dir);
   bool start(QStringList *env = 0);
-  bool isRunning() const;
-  bool normalExit() const;
-  int exitStatus() const;
   QByteArray readStdout();
   QByteArray readStderr();
-  Q_LONG processIdentifier() const;
-
-public slots:
-  void kill() const;
-  void writeToStdin(const QByteArray &buf);
 
 private slots:
-  void receivedStdout(KProcess*, char *buf, int len);
-  void receivedStderr(KProcess*, char *buf, int len);
+  void receivedStandardOutput();
+  void receivedStandardError();
+  void stateChanged(QProcess::ProcessState);
 
 signals:
   void readyReadStdout();
@@ -47,6 +38,8 @@ signals:
 private:
   std::auto_ptr<KProcess> _kproc;
   QByteArray              _stdout, _stderr;
+
+friend class Base;
 };
 
 } // namespace
