@@ -10,12 +10,10 @@
 #define PURL_H
 
 #include "common/common/global.h"
-#if QT_VERSION<0x040000
-#  include <qhttp.h>
-#else
-#  include <QtNetwork/QHttp>
-#  include <QDateTime>
-#endif
+
+#include <QtNetwork/QHttp>
+#include <QDateTime>
+
 #include "log.h"
 #include "common/common/purl_base.h"
 
@@ -46,11 +44,11 @@ class Base
 {
 public:
   Base(const QString &filepath = QString::null);
-  Base(const KURL &url);
+  Base(const KUrl &url);
   bool operator <(const Base &url) const { return _url<url._url; }
   bool operator ==(const Base &url) const;
   bool operator !=(const Base &url) const { return !(_url==url._url); }
-  const KURL &kurl() const { return _url; }
+  const KUrl &kurl() const { return _url; }
   QString pretty() const;
   bool isEmpty() const { return _url.isEmpty(); }
   bool isLocal() const;
@@ -62,7 +60,7 @@ public:
 
 protected:
   bool _relative;
-  KURL _url;
+  KUrl _url;
 
 private:
   bool httpUrlExists(bool *ok = 0) const;
@@ -73,7 +71,7 @@ class Url : public Base
 {
 public:
   Url() {}
-  Url(const KURL &url) : Base(url) {}
+  Url(const KUrl &url) : Base(url) {}
   // add correct extension if filename has no extension
   Url(const Directory &path, const QString &filename, FileType type);
   Url(const Directory &path, const QString &filepath);
@@ -108,9 +106,7 @@ public:
   UrlList() {}
   UrlList(const Url &url) { append(url); }
   UrlList(const QValueList<Url> &list) : QValueList<Url>(list) {}
-#if !defined(NO_KDE)
-  UrlList(const KURL::List &list);
-#endif
+  UrlList(const KUrl::List &list);
 };
 
 //----------------------------------------------------------------------------
