@@ -37,14 +37,15 @@ const Pic::Architecture::Data Pic::Architecture::DATA[Nb_Types] = {
 //  name family_label nbBytesPC nbBytesWord nbBitsRegister registerBankLength
 //  {Code, Cal, UserID, DevId, Conf, EEPROM, DebugVec, HardStack, CalBackup, Program Executive}
 //  properties nominalVdd
-  { "10X", I18N_NOOP("Baseline Family"),  0, 2, 8, 0x020, { 12, 12, 12, 12, 12,  8, 12,  0, 12,  0 },  NoArchitectureProperty, 5.0 }, // 9, 10, 11 or 12-bit program counter
-  { "16X", I18N_NOOP("Midrange Family"), 13, 2, 8, 0x080, { 14, 14, 14, 14, 14,  8, 14,  0, 14,  0 },  MaybeSelfWrite, 5.0 }, // max eeprom: 256 words
+  { "baseline", I18N_NOOP("Baseline Family"),  0, 2, 8, 0x020, { 12, 12, 12, 12, 12,  8, 12,  0, 12,  0 },  NoArchitectureProperty, 5.0 }, // 9, 10, 11 or 12-bit program counter
+  { "midrange", I18N_NOOP("Mid-Range Family"), 13, 2, 8, 0x080, { 14, 14, 14, 14, 14,  8, 14,  0, 14,  0 },  MaybeSelfWrite, 5.0 }, // max eeprom: 256 words
+  { "emidrange", I18N_NOOP("Enhance Mid-Range Family"), 13, 2, 8, 0x080, { 14, 14, 14, 14, 14,  8, 14,  0, 14,  0 },  MaybeSelfWrite, 5.0 }, // max eeprom: 256 words
   { "17C", I18N_NOOP("17C Family"),      16, 2, 8, 0x100, { 16,  0,  0,  0, 16,  8,  0,  0,  0,  0 },  RandomMemoryAccess,     5.0 },
   { "18C", I18N_NOOP("18C Family"),      21, 2, 8, 0x100, { 16,  8,  8,  8,  8,  8, 16,  0,  8,  0 },  PackedHex | RandomMemoryAccess, 5.0 },
   { "18F", I18N_NOOP("18F Family"),      21, 2, 8, 0x100, { 16,  8,  8,  8,  8,  8, 16,  0,  8,  0 },  PackedHex | RandomMemoryAccess | MaybeSelfWrite, 5.0 },
   { "18J", I18N_NOOP("18J Family"),      21, 2, 8, 0x100, { 16,  8,  8,  8,  8,  8, 16,  0,  8,  0 },  PackedHex | RandomMemoryAccess | SelfWrite | JFamily, 3.3 },
 
-  { "24F", I18N_NOOP("24F Family"),      23, 4, 16, 0x800, { 24,  0,  0, 16, 24,  0, 24,  0,  0, 24 }, RandomMemoryAccess | SelfWrite | JFamily, 3.3 },
+  { "24F", I18N_NOOP("24F Family"),      23, 4, 16, 0x800, { 24,  0,  0, 16, 16,  0, 24,  0,  0, 24 }, RandomMemoryAccess | SelfWrite | JFamily, 3.3 },
   { "24H", I18N_NOOP("24H Family"),      23, 4, 16, 0x800, { 24,  0,  8, 16,  8,  0, 24,  0,  0, 24 }, RandomMemoryAccess | SelfWrite | JFamily, 3.3 },
   { "30F", I18N_NOOP("30F Family"),      23, 4, 16, 0xA00, { 24,  0, 24, 16, 16, 16, 24,  0,  0, 24 }, RandomMemoryAccess | SelfWrite, 5.0 }, // dsPIC: eeprom max = 2 kwords = 4 kbytes
   { "33F", I18N_NOOP("33F Family"),      23, 4, 16, 0x800, { 24,  0, 16, 16, 16,  0, 24,  0,  0, 24 }, RandomMemoryAccess | SelfWrite | JFamily, 3.3 }
@@ -75,8 +76,8 @@ const Pic::Feature::Data Pic::Feature::DATA[Nb_Types] = {
 
 //-----------------------------------------------------------------------------
 Pic::Data::Data()
-  : Device::Data(new RegistersData(*this))
 {
+  _registersData.reset(new RegistersData(*this));
   FOR_EACH(ProgVoltageType, type) {
     _voltages[type].min = 0.0;
     _voltages[type].max = 0.0;
@@ -88,17 +89,11 @@ Pic::Data::Data()
     _ranges[type].end = 0;
     _ranges[type].hexFileOffset = 0;
   }
-  _config = new Config(*this);
+  _config.reset(new Config(*this));
   _calibration.opcode = 0;
   _calibration.opcodeMask = 0;
 }
 
-Pic::Data::~Data()
-{
-  delete _config;
-  _config = NULL;
-}
-
 bool Pic::Data::isReadable(MemoryRangeType type) const
 {
   return ( range(type).properties & Programmable );
@@ -146,7 +141,7 @@ QString Pic::Data::fname(Device::Special special) const
       // assume name is of form "NNXN..."
       s.replace(2, 1, "HV");
       break;
-    case Device::Special::Nb_Types: Q_ASSERT(false); break;
+    case Device::Special::Nb_Types: ASSERT(false); break;
   }
   return s;
 }
@@ -159,8 +154,9 @@ bool Pic::Data::matchId(BitValue rawId, Device::IdData &idata) const
     idata.special = it.key();
     BitValue nid = 0x0;
     switch (architecture().type()) {
-      case Architecture::P10X:
-      case Architecture::P16X:
+      case Architecture::Baseline:
+      case Architecture::MidRange:
+      case Architecture::EMidRange:
       case Architecture::P17C:
       case Architecture::P18C:
       case Architecture::P18F:
@@ -184,7 +180,7 @@ bool Pic::Data::matchId(BitValue rawId, Device::IdData &idata) const
         nid = (rawId >> 16).maskWith(0xFFFF);
         idata.revision = rawId.maskWith(0xFFFF); // ??
         break;
-      case Architecture::Nb_Types: Q_ASSERT(false); break;
+      case Architecture::Nb_Types: ASSERT(false); break;
     }
     if ( nid==it.data() ) return true;
   }
@@ -197,8 +193,9 @@ QStringList Pic::Data::idNames(const QMap<QString, Device::IdData> &ids) const
   QMap<QString, Device::IdData>::const_iterator it;
   for (it=ids.begin(); it!=ids.end(); ++it) {
     switch (_architecture.type()) {
-      case Architecture::P10X:
-      case Architecture::P16X:
+      case Architecture::Baseline:
+      case Architecture::MidRange:
+      case Architecture::EMidRange:
       case Architecture::P17C:
       case Architecture::P18C:
       case Architecture::P18F:
@@ -215,7 +212,7 @@ QStringList Pic::Data::idNames(const QMap<QString, Device::IdData> &ids) const
       case Architecture::P33F:
         list += i18n("%1 (rev. %2)").arg(it.key()).arg(toLabel(it.data().revision));
         break;
-      case Architecture::Nb_Types: Q_ASSERT(false); break;
+      case Architecture::Nb_Types: ASSERT(false); break;
     }
   }
   return list;
@@ -229,7 +226,7 @@ bool Pic::Data::isCalibrationAtCodeEnd() const
 
 bool Pic::Data::checkCalibration(const Device::Array &data, QString *message) const
 {
-  Q_ASSERT( nbWords(MemoryRangeType::Cal)==data.count() );
+  ASSERT( nbWords(MemoryRangeType::Cal)==data.count() );
   for (uint i=0; i<data.count(); i++) {
     QString address = toHexLabel(range(MemoryRangeType::Cal).start + i*addressIncrement(MemoryRangeType::Cal), nbCharsAddress());
     if ( data[i]==mask(MemoryRangeType::Cal) ) {
@@ -274,22 +271,23 @@ bool Pic::Data::hasFeature(Feature feature, bool *unknown) const
     case Feature::MotorControl: return registersData().sfrs.contains("PWMCON0");
     case Feature::MotionFeedback: return registersData().sfrs.contains("CAP1CON");
     case Feature::SelfWrite: return _selfWrite;
-    case Feature::Nb_Types: Q_ASSERT(false); break;
+    case Feature::Nb_Types: ASSERT(false); break;
   }
   return false;
 }
 
 Device::Array Pic::Data::gotoInstruction(Address address, bool withPageSelection) const
 {
-  Q_ASSERT( address<addressIncrement(MemoryRangeType::Code)*nbWords(MemoryRangeType::Code) );
+  ASSERT( address<addressIncrement(MemoryRangeType::Code)*nbWords(MemoryRangeType::Code) );
   Device::Array a;
   switch (_architecture.type()) {
-  case Architecture::P10X:
+  case Architecture::Baseline:
     if ( nbWords(MemoryRangeType::Code)>0x1FF && withPageSelection)
       a.append(0x4A3 | (address>0x1FF ? 0x100 : 0x000)); // bsf STATUS,PA0 or bcf STATUS,PA0
     a.append(0xA00 | (address.toUInt() & 0x1FF)); // goto
     break;
-  case Architecture::P16X:
+  case Architecture::MidRange:
+  case Architecture::EMidRange:
     if ( nbWords(MemoryRangeType::Code)>0x7FF && withPageSelection ) {
       if ( address<=0x7FF ) a.append(0x018A); // clrf PCLATH
       else {
@@ -315,7 +313,7 @@ Device::Array Pic::Data::gotoInstruction(Address address, bool withPageSelection
     a.append(0x040000 | (address.toUInt() & 0x00FFFE));
     a.append(0X000000 | (address.toUInt() >> 16));
     break;
-  case Architecture::Nb_Types: Q_ASSERT(false); break;
+  case Architecture::Nb_Types: ASSERT(false); break;
   }
   return a;
 }
@@ -323,8 +321,9 @@ Device::Array Pic::Data::gotoInstruction(Address address, bool withPageSelection
 bool Pic::Data::isGotoInstruction(BitValue instruction) const
 {
   switch (_architecture.type()) {
-  case Architecture::P10X: return ( instruction.maskWith(0xE00)==0xA00 );
-  case Architecture::P16X: return ( instruction.maskWith(0xF800)==0x2800 );
+  case Architecture::Baseline: return ( instruction.maskWith(0xE00)==0xA00 );
+  case Architecture::MidRange:
+  case Architecture::EMidRange: return ( instruction.maskWith(0xF800)==0x2800 );
   case Architecture::P17C: return ( instruction.maskWith(0xE000)==0xC000 );
   case Architecture::P18C:
   case Architecture::P18F:
@@ -333,16 +332,18 @@ bool Pic::Data::isGotoInstruction(BitValue instruction) const
   case Architecture::P24H:
   case Architecture::P30F:
   case Architecture::P33F: return ( instruction.maskWith(0xFF0000)==0x040000 );
-  case Architecture::Nb_Types: Q_ASSERT(false); break;
+  case Architecture::Nb_Types: ASSERT(false); break;
   }
   return false;
 }
 
 uint Pic::Data::nbWordsWriteAlignment(MemoryRangeType type) const
 {
-  if ( type!=MemoryRangeType::Code ) return 1;
+  if (type != MemoryRangeType::Code
+       && type != MemoryRangeType::ProgramExecutive) return 1;
   if (_nbWordsCodeWrite == 0) {
     switch (architecture().type()) {
+      case Architecture::P24F:
       case Architecture::P24H:
       case Architecture::P33F: return 64;
       case Architecture::P30F: return 32; // ?
@@ -353,9 +354,11 @@ uint Pic::Data::nbWordsWriteAlignment(MemoryRangeType type) const
 
 uint Pic::Data::nbWordsRowEraseAlignment(MemoryRangeType type) const
 {
-  if ( type!=MemoryRangeType::Code ) return 1;
+  if (type != MemoryRangeType::Code
+      && type != MemoryRangeType::ProgramExecutive) return 1;
   if (_nbWordsCodeRowErase == 0) {
     switch (architecture().type()) {
+      case Architecture::P24F:
       case Architecture::P24H:
       case Architecture::P33F: return 512;
       default:                 return 16; // default ??
