@@ -67,13 +67,18 @@ const Pic::Config::Data Pic::Config::DATA[] = {
     { "INTXT", I18N_NOOP("Internal oscillator, XT used by USB") },
     { "INTHS", I18N_NOOP("Internal oscillator, HS used by USB") },
 
-    { "EC_CLKOUT_HIGH", I18N_NOOP("External clock with CLKOUT, high power") },
-    { "EC_CLKOUT_MED", I18N_NOOP("External clock with CLKOUT, medium power") },
-    { "EC_CLKOUT_LOW", I18N_NOOP("External clock with CLKOUT, low power") },
-    { "EC_HIGH", I18N_NOOP("External clock, high power") },
-    { "EC_MED", I18N_NOOP("External clock, medium power") },
-    { "EC_LOW", I18N_NOOP("External clock, low power") },
+    { "EC_CLKOUT_HP", I18N_NOOP("External clock with CLKOUT, high power") },
+    { "EC_CLKOUT_MP", I18N_NOOP("External clock with CLKOUT, medium power") },
+    { "EC_CLKOUT_LP", I18N_NOOP("External clock with CLKOUT, low power") },
+    { "EC_HP", I18N_NOOP("External clock, high power") },
+    { "EC_MP", I18N_NOOP("External clock, medium power") },
+    { "EC_LP", I18N_NOOP("External clock, low power") },
+
+    { "HS_HP", I18N_NOOP("HS oscillator, high power") },
+    { "HS_MP", I18N_NOOP("HS oscillator, medium power") },
+
     { 0, 0 } } },
+
     { { "CLKOUTEN", I18N_NOOP("Clock out") }, Toggle, { { 0, 0 } } },
 
   { { "BG",    I18N_NOOP("Bandgap voltage calibration") }, Fixed, {
@@ -97,6 +102,8 @@ const Pic::Config::Data Pic::Config::DATA[] = {
   { { "BORV",  I18N_NOOP("Brown-out reset voltage") }, ValueDouble, {
     { "0", I18N_NOOP("Undefined")  }, { 0, 0 } } },
   { { "LVP",   I18N_NOOP("Low voltage programming")   }, Toggle, { { 0, 0 } } },
+  { { "LPBOR", I18N_NOOP("Low power brown-out reset")   }, Toggle, { { 0, 0 } } },
+  { { "CCP3MX", I18N_NOOP("CCP3 multiplex")           }, Pin, { { 0, 0 } } },
   { { "CCP2MX", I18N_NOOP("CCP2 multiplex")           }, Pin, { { 0, 0 } } },
   { { "CCP1MX", I18N_NOOP("CCP1 multiplex")           }, Pin, { { 0, 0 } } },
   { { "BORSEN", I18N_NOOP("Brown-out reset software") }, Toggle, { { 0, 0 } } },
@@ -148,6 +155,8 @@ const Pic::Config::Data Pic::Config::DATA[] = {
     { "RH7-RH4", I18N_NOOP("PWM multiplexed onto RH7 and RH4") },
     { "RE6-RE5", I18N_NOOP("PWM multiplexed onto RE6 and RE5") },
     { "RH7-RH6", I18N_NOOP("PWM multiplexed onto RH7 and RH6") }, { 0, 0 } } },
+  { { "T3CMX", I18N_NOOP("Timer3 Clock Input mux") }, Pin, { { 0, 0 } } },
+  { { "P2BMX", I18N_NOOP("P2B Input mux") }, Pin, { { 0, 0 } } },
 
   { { "FCKSM", I18N_NOOP("Clock switching mode") }, Fixed, {
     { "Switching off, monitor off", I18N_NOOP("Switching off, monitor off") },
