@@ -42,6 +42,13 @@ void Pic::Memory::fill(BitValue value)
   FOR_EACH(MemoryRangeType, k) fill(k, value);
 }
 
+std::auto_ptr<Pic::Memory> Pic::Memory::clone() const
+{
+  std::auto_ptr<Memory> memory(new Memory(device()));
+  memory->copyFrom(*this);
+  return memory;
+}
+
 void Pic::Memory::copyFrom(MemoryRangeType type, const Memory &memory)
 {
   ASSERT( memory.device().name()==device().name() );
@@ -105,7 +112,7 @@ QString Pic::Memory::findValue(const QString &maskName) const
   BitValue v = word(MemoryRangeType::Config, i).maskWith(mask->value);
   for (uint k=0; k<uint(mask->values.count()); k++)
     if ( v.isInside(mask->values[k].value) ) return mask->values[k].name;
-  Q_ASSERT(false);
+  ASSERT(false);
   return QString::null;
 }
 
@@ -128,14 +135,14 @@ AddressRange Pic::Memory::bootRange() const
     AddressRangeVector rv = protection.extractRanges(mask->values[k].name, MemoryRangeType::Code);
     if ( !rv.isEmpty() ) return rv[0];
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return AddressRange();
 }
 
 AddressRange Pic::Memory::blockRange(uint i) const
 {
   const Protection &protection = device().config().protection();
-  Q_ASSERT( i<protection.nbBlocks() );
+  ASSERT( i<protection.nbBlocks() );
   if ( protection.family()==Protection::CodeGuard && i==1 ) { // general segment
     AddressRange previous = blockRange(0);
     if ( previous.isEmpty() ) previous = bootRange();
@@ -145,7 +152,7 @@ AddressRange Pic::Memory::blockRange(uint i) const
   QString maskName = protection.blockSizeMaskName(i);
   if ( protection.family()==Protection::CodeGuard ) { // secure segment
     QString value = findValue(maskName);
-    Q_ASSERT( !value.isEmpty() );
+    ASSERT( !value.isEmpty() );
     uint size = value.toUInt();
     if ( size==0 ) return AddressRange();
     AddressRange previous = bootRange();
@@ -158,7 +165,7 @@ AddressRange Pic::Memory::blockRange(uint i) const
     AddressRangeVector rv = protection.extractRanges(mask->values[k].name, MemoryRangeType::Code);
     if ( !rv.isEmpty() ) return AddressRange(previous.end + 1, rv[0].end);
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return AddressRange();
 }
 
@@ -207,7 +214,7 @@ AddressRangeVector Pic::Memory::protectedRanges(Protection::Type ptype, MemoryRa
     //qDebug("%s %s", maskName.latin1(), value.latin1());
     if ( !value.isEmpty() ) {
       AddressRangeVector tmp = protection.extractRanges(value, type);
-      Q_ASSERT( tmp.count()==1 );
+      ASSERT( tmp.count()==1 );
       rv.append(tmp[0]);
     }
   }
@@ -292,11 +299,12 @@ void Pic::Memory::checksumCheckFill()
 {
   clear();
   switch (device().architecture().type()) {
-    case Architecture::P10X:
+    case Architecture::Baseline:
       setWord(MemoryRangeType::Code, device().range(MemoryRangeType::Code).start.toUInt(), 0x723);
       setWord(MemoryRangeType::Code, device().range(MemoryRangeType::Code).end.toUInt(), 0x723);
       break;
-    case Architecture::P16X:
+    case Architecture::MidRange:
+    case Architecture::EMidRange:
       if ( device().name()=="16F72" || device().name()=="16F73" || device().name()=="16F74" || device().name()=="16F76" || device().name()=="16F77"
            || device().name()=="16CR73" || device().name()=="16CR74" || device().name()=="16CR76" || device().name()=="16CR77" ) {
         setWord(MemoryRangeType::Code, device().range(MemoryRangeType::Code).start.toUInt(), 0x05E6);
@@ -323,7 +331,7 @@ void Pic::Memory::checksumCheckFill()
       setWord(MemoryRangeType::Code, device().range(MemoryRangeType::Code).start.toUInt(), 0xAAAAAA);
       setWord(MemoryRangeType::Code, device().range(MemoryRangeType::Code).end.toUInt(), 0xAAAAAA);
       break;
-    case Architecture::Nb_Types: Q_ASSERT(false); break;
+    case Architecture::Nb_Types: ASSERT(false); break;
   }
 }
 
@@ -359,8 +367,9 @@ BitValue Pic::Memory::checksum() const
     BitValue v = word(MemoryRangeType::Code, i).maskWith(mask);
     //if ( i==0 || i==device().nbWords(MemoryRangeType::Code)-1 ) qDebug("%s %s", toHexLabel(i, 4).latin1(), toHexLabel(v, 4).latin1());
     switch (device().architecture().type()) {
-      case Architecture::P10X:
-      case Architecture::P16X:
+      case Architecture::Baseline:
+      case Architecture::MidRange:
+      case Architecture::EMidRange:
       case Architecture::P17C:
         if ( rv.contains(i) ) {
           switch (algorithm.type()) {
@@ -368,7 +377,7 @@ BitValue Pic::Memory::checksum() const
             case Checksum::Algorithm::XOR4:   cs += v.XORn(4);  break;
             case Checksum::Algorithm::XNOR7:  cs += v.XNORn(7); break;
             case Checksum::Algorithm::XNOR8:  cs += v.XNORn(8) + (v.XNORn(8) << 8); break;
-            case Checksum::Algorithm::Nb_Types: Q_ASSERT(false); break;
+            case Checksum::Algorithm::Nb_Types: ASSERT(false); break;
           }
         } else cs += v;
         break;
@@ -383,7 +392,7 @@ BitValue Pic::Memory::checksum() const
       case Architecture::P33F:
         cs += v.byte(0) + v.byte(1) + v.byte(2);
         break;
-      case Architecture::Nb_Types: Q_ASSERT(false); break;
+      case Architecture::Nb_Types: ASSERT(false); break;
     }
   }
   //qDebug("after code: %s", toHexLabelAbs(cs).latin1());
@@ -421,7 +430,7 @@ BitValue Pic::Memory::checksum() const
 BitValue Pic::Memory::unprotectedChecksum() const
 {
   const Protection &protection = device().config().protection();
-  std::auto_ptr<Memory> tmp(static_cast<Pic::Memory*>(clone()));
+  const std::auto_ptr<Memory> tmp = clone();
   if ( protection.hasBootBlock() ) tmp->setBootProtection(false, Protection::ProgramProtected);
   if ( protection.nbBlocks()!=0 ) {
     for (uint i=0; i<protection.nbBlocks(); i++)
