@@ -14,24 +14,25 @@
 #include "pic_debug.h"
 
 //-----------------------------------------------------------------------------
-Debugger::DeviceSpecific *Programmer::PicGroup::createDebuggerDeviceSpecific(::Debugger::Base &base) const
+Debugger::DeviceSpecific* Programmer::PicGroup::createDebuggerDeviceSpecific(::Debugger::Base &base) const
 {
   const Pic::Data *data = static_cast<const Pic::Data *>(base.device());
-  if ( data==NULL ) return NULL;
-  switch (data->architecture().type()) {
-    case Pic::Architecture::P10X:
-    case Pic::Architecture::P16X: return new ::Debugger::P16FSpecific(base);
-    case Pic::Architecture::P18C:
-    case Pic::Architecture::P18F:
-    case Pic::Architecture::P18J: return new ::Debugger::P18FSpecific(base);
-    case Pic::Architecture::P24F:
-    case Pic::Architecture::P24H:
-    case Pic::Architecture::P30F:
-    case Pic::Architecture::P33F:
-    case Pic::Architecture::P17C:
-    case Pic::Architecture::Nb_Types: break;
+  if (data != NULL) {
+    switch (data->architecture().type()) {
+      case Pic::Architecture::Baseline:
+      case Pic::Architecture::MidRange:
+      case Pic::Architecture::EMidRange: return new ::Debugger::P16FSpecific(base);
+      case Pic::Architecture::P18C:
+      case Pic::Architecture::P18F:
+      case Pic::Architecture::P18J: return new ::Debugger::P18FSpecific(base);
+      case Pic::Architecture::P24F:
+      case Pic::Architecture::P24H:
+      case Pic::Architecture::P30F:
+      case Pic::Architecture::P33F:
+      case Pic::Architecture::P17C:
+      case Pic::Architecture::Nb_Types: ASSERT(false); break;
+    }
   }
-  Q_ASSERT(false);
   return NULL;
 }
 
@@ -59,7 +60,7 @@ uint Programmer::PicBase::nbSteps(Task task, const Device::MemoryRange& range) c
       uint nb = 0;
       FOR_EACH(Pic::MemoryRangeType, type) {
         if ( type!=Pic::MemoryRangeType::Code && type!=Pic::MemoryRangeType::Eeprom ) continue;
-        if ( !device()->isReadable(type) || !specific()->canReadRange(type) ) continue;
+        if ( !device()->isReadable(type) || !deviceSpecific()->canReadRange(type) ) continue;
         if ( !prange.all() && prange._type!=type ) continue;
         nb += device()->nbWords(type);
       }
@@ -69,7 +70,7 @@ uint Programmer::PicBase::nbSteps(Task task, const Device::MemoryRange& range) c
       uint nb = 0;
       FOR_EACH(Pic::MemoryRangeType, type) {
         if ( type!=Pic::MemoryRangeType::Code && type!=Pic::MemoryRangeType::Eeprom ) continue;
-        if ( !device()->isWritable(type) || !specific()->canWriteRange(type) ) continue;
+        if ( !device()->isWritable(type) || !deviceSpecific()->canWriteRange(type) ) continue;
         if ( !prange.all() && prange._type!=type ) continue;
         nb += device()->nbWords(type);
         if ( readConfigEntry(Config::VerifyAfterProgram).toBool() ) nb += device()->nbWords(type);
@@ -78,7 +79,7 @@ uint Programmer::PicBase::nbSteps(Task task, const Device::MemoryRange& range) c
     }
     case Task::Nb_Types: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return 0;
 }
 
@@ -101,7 +102,7 @@ bool Programmer::PicBase::initProgramming(Task)
     }
   }
   if ( vdd()!=UNKNOWN_VOLTAGE ) {
-    Q_ASSERT( type!=Pic::Vpp );
+    ASSERT( type!=Pic::Vpp );
     const Pic::VoltageData &tvdd = device()->voltage(type);
     if ( vdd()<tvdd.min ) {
       if ( type==Pic::VddBulkErase && device()->voltage(Pic::VddWrite).min!=tvdd.min )
@@ -123,10 +124,10 @@ bool Programmer::PicBase::initProgramming(Task)
     }
   }
 */
-  if ( specific()->canReadRange(Pic::MemoryRangeType::Config) ) {
+  if ( deviceSpecific()->canReadRange(Pic::MemoryRangeType::Config) ) {
     // read config
     Device::Array data;
-    if ( !specific()->read(Pic::MemoryRangeType::Config, data, 0) ) return false;
+    if ( !deviceSpecific()->read(Pic::MemoryRangeType::Config, data, 0) ) return false;
     _deviceMemory->setArray(Pic::MemoryRangeType::Config, data);
     _hasProtectedCode = _deviceMemory->isProtected(Pic::Protection::ProgramProtected, Pic::MemoryRangeType::Code);
     _hasProtectedEeprom = _deviceMemory->isProtected(Pic::Protection::ProgramProtected, Pic::MemoryRangeType::Eeprom);
@@ -157,12 +158,13 @@ bool Programmer::PicBase::preserveEeprom()
 BitValue Programmer::PicBase::readDeviceId()
 {
   Device::Array data;
-  if ( !specific()->read(Pic::MemoryRangeType::DeviceId, data, 0) ) return 0;
-  Q_ASSERT( data.count()!=0 );
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::DeviceId, data, 0) ) return 0;
+  ASSERT( data.count()!=0 );
   BitValue id = 0x0;
   switch (device()->architecture().type()) {
-    case Pic::Architecture::P10X:
-    case Pic::Architecture::P16X:
+    case Pic::Architecture::Baseline:
+    case Pic::Architecture::MidRange:
+    case Pic::Architecture::EMidRange:
     case Pic::Architecture::P17C: id = data[0]; break;
     case Pic::Architecture::P18C:
     case Pic::Architecture::P18F:
@@ -171,14 +173,14 @@ BitValue Programmer::PicBase::readDeviceId()
     case Pic::Architecture::P24H:
     case Pic::Architecture::P30F:
     case Pic::Architecture::P33F: id = data[1] | (data[0] << 16); break;
-    case Pic::Architecture::Nb_Types: Q_ASSERT(false); break;
+    case Pic::Architecture::Nb_Types: ASSERT(false); break;
   }
   return id;
 }
 
 bool Programmer::PicBase::verifyDeviceId()
 {
-  if ( !specific()->canReadRange(Pic::MemoryRangeType::DeviceId ) ) return true;
+  if ( !deviceSpecific()->canReadRange(Pic::MemoryRangeType::DeviceId ) ) return true;
   if ( !device()->isReadable(Pic::MemoryRangeType::DeviceId) ) {
     log(Log::LineType::Information, i18n("Device not autodetectable: continuing with the specified device name \"%1\"...").arg(device()->name()));
     return true;
@@ -230,22 +232,22 @@ QString Programmer::PicBase::prettyCalibration(const Device::Array &data) const
 bool Programmer::PicBase::readCalibration()
 {
   if ( device()->isReadable(Pic::MemoryRangeType::Cal) ) {
-    if ( !specific()->canReadRange(Pic::MemoryRangeType::Cal) ) {
+    if ( !deviceSpecific()->canReadRange(Pic::MemoryRangeType::Cal) ) {
       log(Log::LineType::Warning, i18n("Osccal cannot be read by the selected programmer"));
       return true;
     }
     Device::Array data;
-    if ( !specific()->read(Pic::MemoryRangeType::Cal, data, 0) ) return false;
+    if ( !deviceSpecific()->read(Pic::MemoryRangeType::Cal, data, 0) ) return false;
     _deviceMemory->setArray(Pic::MemoryRangeType::Cal, data);
     log(Log::DebugLevel::Normal, QString("  Read osccal: %1").arg(prettyCalibration(data)));
     QString message;
     if ( !device()->checkCalibration(data, &message) ) log(Log::LineType::Warning, "  " + message);
     if ( device()->isReadable(Pic::MemoryRangeType::CalBackup) ) {
-      if ( !specific()->canReadRange(Pic::MemoryRangeType::CalBackup) ) {
+      if ( !deviceSpecific()->canReadRange(Pic::MemoryRangeType::CalBackup) ) {
         log(Log::LineType::Warning, i18n("Osccal backup cannot be read by the selected programmer"));
         return true;
       }
-      if ( !specific()->read(Pic::MemoryRangeType::CalBackup, data, 0) ) return false;
+      if ( !deviceSpecific()->read(Pic::MemoryRangeType::CalBackup, data, 0) ) return false;
       _deviceMemory->setArray(Pic::MemoryRangeType::CalBackup, data);
       log(Log::DebugLevel::Normal, QString("  Read osccal backup: %1").arg(prettyCalibration(data)));
       if ( !device()->checkCalibration(data, &message) ) log(Log::LineType::Warning, "  " + message);
@@ -256,28 +258,28 @@ bool Programmer::PicBase::readCalibration()
 
 bool Programmer::PicBase::restoreCalibration()
 {
-  if ( !specific()->canReadRange(Pic::MemoryRangeType::Cal) || !specific()->canWriteRange(Pic::MemoryRangeType::Cal) ) return true;
+  if ( !deviceSpecific()->canReadRange(Pic::MemoryRangeType::Cal) || !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::Cal) ) return true;
   if ( !device()->isWritable(Pic::MemoryRangeType::Cal) ) return true;
   Device::Array data = _deviceMemory->arrayForWriting(Pic::MemoryRangeType::Cal);
   Device::Array bdata = _deviceMemory->arrayForWriting(Pic::MemoryRangeType::CalBackup);
-  if ( device()->isReadable(Pic::MemoryRangeType::CalBackup) && specific()->canReadRange(Pic::MemoryRangeType::CalBackup) ) {
+  if ( device()->isReadable(Pic::MemoryRangeType::CalBackup) && deviceSpecific()->canReadRange(Pic::MemoryRangeType::CalBackup) ) {
     if ( !device()->checkCalibration(data) && device()->checkCalibration(bdata) ) {
       log(Log::LineType::Information, i18n("  Replace invalid osccal with backup value."));
       data = bdata;
     }
   }
   Device::Array cdata;
-  if ( !specific()->read(Pic::MemoryRangeType::Cal, cdata, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::Cal, cdata, 0) ) return false;
   if ( cdata==data ) {
     log(Log::LineType::Information, i18n("  Osccal is unchanged."));
     return true;
   }
   if ( !programRange(Pic::MemoryRangeType::Cal, data) ) return false;
-  if ( !specific()->read(Pic::MemoryRangeType::Cal, cdata, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::Cal, cdata, 0) ) return false;
   if ( cdata==data ) log(Log::LineType::Information, i18n("  Osccal has been preserved."));
 
   if ( !device()->isWritable(Pic::MemoryRangeType::CalBackup) || !device()->checkCalibration(bdata) ) return true;
-  if ( !specific()->read(Pic::MemoryRangeType::CalBackup, cdata, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::CalBackup, cdata, 0) ) return false;
   if ( cdata.count()==0 ) {
     log(Log::LineType::Warning, i18n("Osccal backup cannot be read by selected programmer"));
     return true;
@@ -287,20 +289,20 @@ bool Programmer::PicBase::restoreCalibration()
     return true;
   }
   if ( !programRange(Pic::MemoryRangeType::CalBackup, bdata) ) return false;
-  if ( !specific()->read(Pic::MemoryRangeType::CalBackup, cdata, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::CalBackup, cdata, 0) ) return false;
   if ( cdata==bdata ) log(Log::LineType::Information, i18n("  Osccal backup has been preserved."));
   return true;
 }
 
 bool Programmer::PicBase::restoreBandGapBits()
 {
-  if ( !specific()->canReadRange(Pic::MemoryRangeType::Config) ) return true;
+  if ( !deviceSpecific()->canReadRange(Pic::MemoryRangeType::Config) ) return true;
   bool hasProtectedBits = false;
   for (uint i=0; i<device()->nbWords(Pic::MemoryRangeType::Config); i++)
     if ( device()->config()._words[i].pmask!=0 ) hasProtectedBits = true;
   if ( !hasProtectedBits ) return true;
   Device::Array cdata;
-  if ( !specific()->read(Pic::MemoryRangeType::Config, cdata, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::Config, cdata, 0) ) return false;
   Device::Array data = _deviceMemory->arrayForWriting(Pic::MemoryRangeType::Config);
   for (uint i=0; i<cdata.count(); i++) {
     BitValue pmask = device()->config()._words[i].pmask;
@@ -308,24 +310,24 @@ bool Programmer::PicBase::restoreBandGapBits()
     cdata[i] = cdata[i].clearMaskBits(pmask);
     cdata[i] |= data[i].maskWith(pmask);
   }
-  if ( !specific()->canWriteRange(Pic::MemoryRangeType::Config) ) {
+  if ( !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::Config) ) {
     log(Log::LineType::Warning, i18n("Could not restore band gap bits because programmer does not support writing config bits."));
     return true;
   }
   log(Log::DebugLevel::Normal, QString("  Write config with band gap bits: %2").arg(toHexLabel(cdata[0], device()->nbCharsWord(Pic::MemoryRangeType::Config))));
   if ( !programRange(Pic::MemoryRangeType::Config, cdata) ) return false;
-  if ( !specific()->read(Pic::MemoryRangeType::Config, data, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::Config, data, 0) ) return false;
   if ( data==cdata ) log(Log::LineType::Information, i18n("  Band gap bits have been preserved."));
   return true;
 }
 
 bool Programmer::PicBase::eraseAll()
 {
-  if ( !(specific()->capabilities() & CanEraseAll) ) {
+  if ( !(deviceSpecific()->capabilities() & CanEraseAll) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer does not support erasing the whole device."));
     return false;
   }
-  if ( !specific()->erase(_hasProtectedCode || _hasProtectedEeprom) ) return false;
+  if ( !deviceSpecific()->erase(_hasProtectedCode || _hasProtectedEeprom) ) return false;
   if ( !restoreCalibration() ) return false;
   return true;
 }
@@ -369,7 +371,7 @@ bool Programmer::PicBase::eraseRange(Pic::MemoryRangeType type)
 
 bool Programmer::PicBase::internalEraseRange(Pic::MemoryRangeType type)
 {
-  if ( !specific()->canEraseRange(type) && !(specific()->capabilities() & CanEraseAll) ) {
+  if ( !deviceSpecific()->canEraseRange(type) && !(deviceSpecific()->capabilities() & CanEraseAll) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer does not support erasing neither the specified range nor the whole device."));
     return false;
   }
@@ -381,16 +383,16 @@ bool Programmer::PicBase::internalEraseRange(Pic::MemoryRangeType type)
     log(Log::LineType::SoftError, i18n("Cannot erase protected data EEPROM. Consider erasing the whole chip."));
     return false;
   }
-  if ( specific()->canEraseRange(type) ) return specific()->eraseRange(type);
+  if ( deviceSpecific()->canEraseRange(type) ) return deviceSpecific()->eraseRange(type);
   bool softErase = true;
-  if ( type!=Pic::MemoryRangeType::Code && (!specific()->canReadRange(Pic::MemoryRangeType::Code)
-       || !specific()->canWriteRange(Pic::MemoryRangeType::Code)) ) softErase = false;
-  if ( type!=Pic::MemoryRangeType::Eeprom && (!specific()->canReadRange(Pic::MemoryRangeType::Eeprom)
-       || !specific()->canWriteRange(Pic::MemoryRangeType::Eeprom)) ) softErase = false;
-  if ( type!=Pic::MemoryRangeType::Config && (!specific()->canReadRange(Pic::MemoryRangeType::Config)
-       || !specific()->canWriteRange(Pic::MemoryRangeType::Config)) ) softErase = false;
-  if ( type!=Pic::MemoryRangeType::UserId && (!specific()->canReadRange(Pic::MemoryRangeType::UserId)
-       || !specific()->canWriteRange(Pic::MemoryRangeType::UserId)) ) softErase = false;
+  if ( type!=Pic::MemoryRangeType::Code && (!deviceSpecific()->canReadRange(Pic::MemoryRangeType::Code)
+       || !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::Code)) ) softErase = false;
+  if ( type!=Pic::MemoryRangeType::Eeprom && (!deviceSpecific()->canReadRange(Pic::MemoryRangeType::Eeprom)
+       || !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::Eeprom)) ) softErase = false;
+  if ( type!=Pic::MemoryRangeType::Config && (!deviceSpecific()->canReadRange(Pic::MemoryRangeType::Config)
+       || !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::Config)) ) softErase = false;
+  if ( type!=Pic::MemoryRangeType::UserId && (!deviceSpecific()->canReadRange(Pic::MemoryRangeType::UserId)
+       || !deviceSpecific()->canWriteRange(Pic::MemoryRangeType::UserId)) ) softErase = false;
   if ( !softErase ) {
     log(Log::LineType::SoftError, i18n("Cannot erase specified range because of programmer limitations."));
     return false;
@@ -402,7 +404,7 @@ bool Programmer::PicBase::internalEraseRange(Pic::MemoryRangeType type)
   if ( type!=Pic::MemoryRangeType::Code && !preserveCode() ) return false;
   if ( type!=Pic::MemoryRangeType::Eeprom && !preserveEeprom() ) return false;
   if ( type!=Pic::MemoryRangeType::UserId && !readRange(Pic::MemoryRangeType::UserId, _deviceMemory, 0) ) return false;
-  specific()->erase(_hasProtectedCode || _hasProtectedEeprom);
+  deviceSpecific()->erase(_hasProtectedCode || _hasProtectedEeprom);
   if ( type!=Pic::MemoryRangeType::Code && !programAndVerifyRange(Pic::MemoryRangeType::Code, *_deviceMemory) ) return false;
   if ( type!=Pic::MemoryRangeType::Eeprom && !programAndVerifyRange(Pic::MemoryRangeType::Eeprom, *_deviceMemory) ) return false;
   if ( type!=Pic::MemoryRangeType::UserId && !programAndVerifyRange(Pic::MemoryRangeType::UserId, *_deviceMemory) ) return false;
@@ -413,7 +415,7 @@ bool Programmer::PicBase::internalEraseRange(Pic::MemoryRangeType type)
 //-----------------------------------------------------------------------------
 bool Programmer::PicBase::readSingle(Pic::MemoryRangeType type, Pic::Memory &memory)
 {
-  if ( !specific()->canReadRange(type) ) {
+  if ( !deviceSpecific()->canReadRange(type) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer cannot read the specified memory range."));
     return false;
   }
@@ -427,7 +429,7 @@ bool Programmer::PicBase::readSingle(Pic::MemoryRangeType type, Pic::Memory &mem
 bool Programmer::PicBase::readRange(Pic::MemoryRangeType type, Pic::Memory *memory, const VerifyData *vd)
 {
   if ( !device()->isReadable(type) ) return true;
-  if ( !specific()->canReadRange(type) ) {
+  if ( !deviceSpecific()->canReadRange(type) ) {
     log(Log::LineType::Information, i18n("The selected programmer cannot read %1: operation skipped.").arg(type.label()));
     return true;
   }
@@ -445,7 +447,7 @@ bool Programmer::PicBase::readRange(Pic::MemoryRangeType type, Pic::Memory *memo
     ASSERT(memory);
   }
   Device::Array data;
-  if (!specific()->read(type, data, vdata.get())) return false;
+  if (!deviceSpecific()->read(type, data, vdata.get())) return false;
   if (memory != NULL) memory->setArray(type, data);
   return true;
 }
@@ -505,7 +507,7 @@ bool Programmer::PicBase::internalRead(Device::Memory *memory, const Device::Mem
 //-----------------------------------------------------------------------------
 bool Programmer::PicBase::programSingle(Pic::MemoryRangeType type, const Pic::Memory &memory)
 {
-  if ( !specific()->canWriteRange(type) ) {
+  if ( !deviceSpecific()->canWriteRange(type) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer cannot read the specified memory range."));
     return false;
   }
@@ -517,16 +519,16 @@ bool Programmer::PicBase::programRange(Pic::MemoryRangeType mtype, const Device:
   log(Log::LineType::Information, i18n("  Write memory: %1").arg(mtype.label()));
   bool only = ( readConfigEntry(Config::OnlyProgramNonMask).toBool()
                 && (mtype==Pic::MemoryRangeType::Code || mtype==Pic::MemoryRangeType::Eeprom) );
-  return specific()->write(mtype, data, !only);
+  return deviceSpecific()->write(mtype, data, !only);
 }
 
 bool Programmer::PicBase::programAndVerifyRange(Pic::MemoryRangeType type, const Pic::Memory &memory)
 {
-  if ( !device()->isWritable(type) || !specific()->canWriteRange(type) ) return true;
+  if ( !device()->isWritable(type) || !deviceSpecific()->canWriteRange(type) ) return true;
   Device::Array data = memory.arrayForWriting(type);
   if ( !programRange(type, data) ) return false;
   if ( !readConfigEntry(Config::VerifyAfterProgram).toBool() ) return true;
-  if ( !specific()->canReadRange(type) ) return true;
+  if ( !deviceSpecific()->canReadRange(type) ) return true;
   VerifyActions actions = IgnoreProtectedVerify;
   if ( type==Pic::MemoryRangeType::Code && readConfigEntry(Config::OnlyVerifyProgrammed).toBool() ) actions |= OnlyProgrammedVerify;
   VerifyData vdata(actions, memory);
@@ -565,7 +567,7 @@ bool Programmer::PicBase::checkProgram(const Device::Memory &)
 bool Programmer::PicBase::internalProgram(const Device::Memory &memory, const Device::MemoryRange &range)
 {
   if ( !initProgramming(Task::Erase) ) return false;
-  std::auto_ptr<Pic::Memory> pmemory(static_cast<Pic::Memory*>(memory.clone()));
+  std::auto_ptr<Pic::Memory> pmemory = static_cast<const Pic::Memory&>(memory).clone();
   prepareMemoryForWriting(*pmemory, true);
 
   // blank check if OTP device
@@ -594,16 +596,16 @@ bool Programmer::PicBase::internalProgram(const Device::Memory &memory, const De
       log(Log::LineType::SoftError, i18n("This memory range is programming protected."));
       return false;
     }
-    if ( specific()->canEraseRange(type) ) {
-      if ( !(specific()->capabilities() & EmulatedErase) && !eraseRange(type) ) return false;
+    if ( deviceSpecific()->canEraseRange(type) ) {
+      if ( !(deviceSpecific()->capabilities() & EmulatedErase) && !eraseRange(type) ) return false;
     } else log(Log::LineType::Warning, i18n("The range cannot be erased first by the selected programmer so programming may fail..."));
     ok = programRange(type, pmemory->arrayForWriting(type));
     VerifyData vdata(NormalVerify, *pmemory);
     if (ok) ok = readRange(type, 0, &vdata);
   } else {
     if ( !eprom ) {
-      if ( specific()->capabilities() & CanEraseAll ) {
-        if ( !(specific()->capabilities() & EmulatedErase) ) {
+      if ( deviceSpecific()->capabilities() & CanEraseAll ) {
+        if ( !(deviceSpecific()->capabilities() & EmulatedErase) ) {
           log(Log::LineType::Information, i18n("  Erasing device"));
           ok = ( !readConfigEntry(Config::PreserveEeprom).toBool() || preserveEeprom() );
           if (ok) ok = eraseAll();
@@ -631,18 +633,18 @@ bool Programmer::PicBase::tryProgramCalibration(const Device::Array &data, bool
 {
   log(Log::LineType::Information, i18n("  Write memory: %1").arg(Pic::MemoryRangeType(Pic::MemoryRangeType::Cal).label()));
   success = true;
-  if ( !specific()->write(Pic::MemoryRangeType::Cal, data, true) ) return false;
+  if ( !deviceSpecific()->write(Pic::MemoryRangeType::Cal, data, true) ) return false;
   Device::Array read;
-  if ( !specific()->read(Pic::MemoryRangeType::Cal, read, 0) ) return false;
+  if ( !deviceSpecific()->read(Pic::MemoryRangeType::Cal, read, 0) ) return false;
   for (uint i=0; i<data.count(); i++)
     if ( data[i]!=read[i] ) success = false;
   if ( !success ) return true;
   if ( device()->isWritable(Pic::MemoryRangeType::CalBackup) ) {
-    if ( !specific()->read(Pic::MemoryRangeType::CalBackup, read, 0) ) return false;
+    if ( !deviceSpecific()->read(Pic::MemoryRangeType::CalBackup, read, 0) ) return false;
     if ( device()->checkCalibration(read) ) return true; // do not overwrite correct backup value
     log(Log::LineType::Information, i18n("  Write memory: %1").arg(Pic::MemoryRangeType(Pic::MemoryRangeType::CalBackup).label()));
-    if ( !specific()->write(Pic::MemoryRangeType::CalBackup, data, true) ) return false;
-    if ( !specific()->read(Pic::MemoryRangeType::CalBackup, read, 0) ) return false;
+    if ( !deviceSpecific()->write(Pic::MemoryRangeType::CalBackup, data, true) ) return false;
+    if ( !deviceSpecific()->read(Pic::MemoryRangeType::CalBackup, read, 0) ) return false;
     for (uint i=0; i<data.count(); i++)
       if ( data[i]!=read[i] ) success = false;
   }
@@ -653,7 +655,7 @@ bool Programmer::PicBase::internalProgramCalibration(const Device::Array &data)
 {
   if ( !initProgramming(Task::Write) ) return false;
   // try without erase
-  bool success;
+  bool success = false;
   if ( !tryProgramCalibration(data, success) ) return false;
   if (success) return true;
   if ( !askContinue(i18n("Programming calibration data needs a chip erase. Continue anyway?")) ) {
@@ -661,7 +663,7 @@ bool Programmer::PicBase::internalProgramCalibration(const Device::Array &data)
     return false;
   }
   log(Log::LineType::Information, i18n("  Erasing device"));
-  bool ok = specific()->erase(_hasProtectedCode || _hasProtectedEeprom);
+  const bool ok = deviceSpecific()->erase(_hasProtectedCode || _hasProtectedEeprom);
   if ( !restoreBandGapBits() ) return false;
   if ( !ok ) return false;
   // retry
