@@ -9,34 +9,34 @@
 #include "pic_prog_specific.h"
 
 //-----------------------------------------------------------------------------
-uint Programmer::PicDeviceSpecific::findNonMaskStart(Pic::MemoryRangeType type, const Device::Array &data) const
+uint Programmer::PicSpecific::findNonMaskStart(Pic::MemoryRangeType type, const Device::Array &data) const
 {
   uint start = 0;
   for (; start<data.count(); start++)
     if ( data[start]!=device().mask(type) ) break;
-  const_cast<PicDeviceSpecific *>(this)->log(Log::DebugLevel::Normal, QString("start before align: %1").arg(start));
+  const_cast<PicSpecific *>(this)->log(Log::DebugLevel::Normal, QString("start before align: %1").arg(start));
   uint align = device().nbWordsWriteAlignment(type);
   start -= start % align;
-  const_cast<PicDeviceSpecific *>(this)->log(Log::DebugLevel::Normal, QString("start after align: %1 (align=%2)").arg(start).arg(align));
+  const_cast<PicSpecific *>(this)->log(Log::DebugLevel::Normal, QString("start after align: %1 (align=%2)").arg(start).arg(align));
   return start;
 }
 
-uint Programmer::PicDeviceSpecific::findNonMaskEnd(Pic::MemoryRangeType type, const Device::Array &data) const
+uint Programmer::PicSpecific::findNonMaskEnd(Pic::MemoryRangeType type, const Device::Array &data) const
 {
-  Q_ASSERT(data.count() > 0);
+  ASSERT(data.count() > 0);
   uint end = data.count() - 1;
   for (; end>0; end--)
     if ( data[end]!=device().mask(type) ) break;
-  const_cast<PicDeviceSpecific *>(this)->log(Log::DebugLevel::Normal, QString("end before align: %1").arg(end));
+  const_cast<PicSpecific *>(this)->log(Log::DebugLevel::Normal, QString("end before align: %1").arg(end));
   uint align = device().nbWordsWriteAlignment(type);
   if ( (end+1) % align ) end += align - (end+1) % align;
   // this can happen when the last word is calibration...
   if (end >= data.count()) end = data.count() - 1;
-  const_cast<PicDeviceSpecific *>(this)->log(Log::DebugLevel::Normal, QString("end after align: %1 (align=%2)").arg(end).arg(align));
+  const_cast<PicSpecific *>(this)->log(Log::DebugLevel::Normal, QString("end after align: %1 (align=%2)").arg(end).arg(align));
   return end;
 }
 
-Device::Array Programmer::PicDeviceSpecific::prepareCodeRange(const Device::Array &data, bool force, uint &wordOffset)
+Device::Array Programmer::PicSpecific::prepareCodeRange(const Device::Array &data, bool force, uint &wordOffset)
 {
   wordOffset = (force || !(capabilities() & VariableOffsetReadWrite) ? 0 : findNonMaskStart(Pic::MemoryRangeType::Code, data));
   uint nbWords = 0;
@@ -55,7 +55,7 @@ Device::Array Programmer::PicDeviceSpecific::prepareCodeRange(const Device::Arra
   return data.mid(wordOffset, nbWords);
 }
 
-bool Programmer::PicDeviceSpecific::read(Pic::MemoryRangeType type, Device::Array &data, const VerifyData *vdata)
+bool Programmer::PicSpecific::read(Pic::MemoryRangeType type, Device::Array &data, const VerifyData *vdata)
 {
   setPowerOn();
   bool ok = internalRead(type, data, vdata);
@@ -63,7 +63,7 @@ bool Programmer::PicDeviceSpecific::read(Pic::MemoryRangeType type, Device::Arra
   return ok;
 }
 
-bool Programmer::PicDeviceSpecific::internalRead(Pic::MemoryRangeType type, Device::Array &data, const VerifyData *vdata)
+bool Programmer::PicSpecific::internalRead(Pic::MemoryRangeType type, Device::Array &data, const VerifyData *vdata)
 {
   if (vdata == NULL) {
     data.resize(device().nbWords(type));
@@ -78,65 +78,94 @@ bool Programmer::PicDeviceSpecific::internalRead(Pic::MemoryRangeType type, Devi
   }
 }
 
-bool Programmer::PicDeviceSpecific::write(Pic::MemoryRangeType type, const Device::Array &data, bool force)
+bool Programmer::PicSpecific::write(Pic::MemoryRangeType type, const Device::Array &data, bool force)
 {
   setPowerOn();
-  bool ok = internalWrite(type, data, force);
+  const bool ok = internalWrite(type, data, force);
   setPowerOff();
   return ok;
 }
 
-bool Programmer::PicDeviceSpecific::internalWrite(Pic::MemoryRangeType type, const Device::Array &data, bool force)
+bool Programmer::PicSpecific::internalWrite(Pic::MemoryRangeType type, const Device::Array &data, bool force)
 {
-  if ( device().architecture()==Pic::Architecture::P18J && !(capabilities() & VariableOffsetReadWrite) )
-    qFatal("Variable-offset-read/write capability is needed for 18J family...");
-  Q_ASSERT( data.count()==device().nbWords(type) );
+  ASSERT(data.count() == device().nbWords(type));
 
-  uint nb = device().nbWordsWriteAlignment(Pic::MemoryRangeType::Code);
-  if ( device().architecture()==Pic::Architecture::P18J && type==Pic::MemoryRangeType::Config ) {
-    Q_ASSERT( data.count()%2==0 );
-    int delta = nb - data.count()/2; // config memory words contains 1 byte
-    Q_ASSERT( delta>=0 );
-    Device::Array rdata(delta);
-    uint wordOffset = device().nbWords(Pic::MemoryRangeType::Code) - delta;
-    if ( !doRead(Pic::MemoryRangeType::Code, wordOffset, rdata, 0) ) return false;
-    Device::Array pdata(nb);
-    for (uint i=0; i<uint(delta); i++) pdata[i] = rdata[i];
-    for (uint i=delta; i<nb; i++) {
-      pdata[i] = data[2*(i-delta)];
-      pdata[i] |= data[2*(i-delta)+1] << 8;
+  const bool configInCodeArea = device().architecture() == Pic::Architecture::P18J
+    || device().architecture() == Pic::Architecture::P24F;
+  if (configInCodeArea
+      && !(capabilities() & VariableOffsetReadWrite)) {
+    qFatal("Variable-offset-read/write capability is needed for 18J and 24F families...");
+  }
+  
+  if (type == Pic::MemoryRangeType::Config
+      && configInCodeArea) {
+    const uint align = device().nbWordsWriteAlignment(Pic::MemoryRangeType::Code);
+    int delta = align;
+    if (device().architecture() == Pic::Architecture::P18J) {
+      ASSERT(data.count()%2 == 0);
+      delta -= data.count()/2; // config memory words contains 1 byte
+    } else {
+      delta -= data.count();
     }
+    ASSERT(delta >= 0);
+    Device::Array rdata(delta);
+    const uint wordOffset = device().nbWords(Pic::MemoryRangeType::Code) - delta;
+    _base.progressMonitor().addTaskTotalProgress(rdata.size());
+    if ( !doRead(Pic::MemoryRangeType::Code, wordOffset, rdata, 0) ) return false;
+    Device::Array pdata(align);
+    for (uint i=0; i<uint(delta); i++) pdata[i] = rdata[i];
+    for (uint i=delta; i<align; i++) {
+      if (device().architecture() == Pic::Architecture::P18J) {
+        pdata[i] = data[2*(i-delta)];
+        pdata[i] |= data[2*(i-delta)+1] << 8;
+      } else {
+	pdata[i] = data[i-delta];
+      }
+    }
+    _base.progressMonitor().addTaskTotalProgress(pdata.size());
     return doWrite(Pic::MemoryRangeType::Code, wordOffset, pdata, true);
   }
 
   if ( type!=Pic::MemoryRangeType::Code ) return doWrite(type, 0, data, true);
 
-  uint wordOffset;
+  uint wordOffset = 0;
   Device::Array pdata = prepareCodeRange(data, force, wordOffset);
-  if ( device().architecture()==Pic::Architecture::P18J ) {
-    uint end = wordOffset + pdata.size();
-    if ( end>=device().nbWords(Pic::MemoryRangeType::Code) ) {
+  if (configInCodeArea) {
+    const uint end = wordOffset + pdata.size();
+    if (end >= device().nbWords(Pic::MemoryRangeType::Code)) {
       Device::Array rdata(device().nbWords(Pic::MemoryRangeType::Config));
+      _base.progressMonitor().addTaskTotalProgress(rdata.size());
       if ( !doRead(Pic::MemoryRangeType::Code, device().nbWords(Pic::MemoryRangeType::Code), rdata, 0) ) return false;
-      uint n = rdata.count() / 2;
-      for (uint i=0; i<n; i++) {
-        pdata[pdata.size() - n + i] = rdata[2*i];
-        pdata[pdata.size() - n + i] |= rdata[2*i+1] << 8;
+      int delta = pdata.size();
+      if (device().architecture() == Pic::Architecture::P18J) {
+	ASSERT(data.count()%2 == 0);
+	delta -= rdata.count()/2;
+      } else {
+	delta -= rdata.count();
+      }
+      ASSERT(delta >= 0);
+      for (uint i=delta; i<pdata.size(); i++) {
+	if (device().architecture() == Pic::Architecture::P18J) {
+          pdata[i] = rdata[2*(i-delta)];
+          pdata[i] |= rdata[2*(i-delta)+1] << 8;
+        } else {
+	  pdata[i] = rdata[i-delta];
+	}
       }
     }
   }
   return doWrite(type, wordOffset, pdata, force);
 }
 
-bool Programmer::PicDeviceSpecific::erase(bool isProtected)
+bool Programmer::PicSpecific::erase(bool isProtected)
 {
   setPowerOn();
-  bool ok = internalErase(isProtected);
+  const bool ok = internalErase(isProtected);
   setPowerOff();
   return ok;
 }
 
-bool Programmer::PicDeviceSpecific::internalErase(bool isProtected)
+bool Programmer::PicSpecific::internalErase(bool isProtected)
 {
   if ( device().architecture()==Pic::Architecture::P18J ) { // ### also true for others ?
     Device::Array data(device().nbWords(Pic::MemoryRangeType::Config));
@@ -146,15 +175,15 @@ bool Programmer::PicDeviceSpecific::internalErase(bool isProtected)
   return doErase(isProtected);
 }
 
-bool Programmer::PicDeviceSpecific::eraseRange(Pic::MemoryRangeType type)
+bool Programmer::PicSpecific::eraseRange(Pic::MemoryRangeType type)
 {
   setPowerOn();
-  bool ok = doEraseRange(type);
+  const bool ok = doEraseRange(type);
   setPowerOff();
   return ok;
 }
 
-bool Programmer::PicDeviceSpecific::doEmulatedEraseRange(Pic::MemoryRangeType type)
+bool Programmer::PicSpecific::doEmulatedEraseRange(Pic::MemoryRangeType type)
 {
   Pic::Memory memory(device());
   if ( !doWrite(type, 0, memory.arrayForWriting(type), true) ) return false;
@@ -168,7 +197,7 @@ bool Programmer::PicDeviceSpecific::doEmulatedEraseRange(Pic::MemoryRangeType ty
 bool Programmer::PicHardware::compareWords(Pic::MemoryRangeType type, uint index, BitValue v, BitValue d, Programmer::VerifyActions actions)
 {
   if ( v==d ) return true;
-  uint inc = device().addressIncrement(type);
+  const uint inc = device().addressIncrement(type);
   Address address = device().range(type).start + inc * index;
   if ( actions & ::Programmer::BlankCheckVerify )
     log(Log::LineType::SoftError, i18n("Device memory is not blank (in %1 at address %2: reading %3 and expecting %4).")
@@ -186,7 +215,7 @@ bool Programmer::PicHardware::verifyWord(uint i, BitValue word, Pic::MemoryRange
   BitValue v = static_cast<const Pic::Memory &>(vdata.memory()).normalizedWord(type, i);
   BitValue d = static_cast<const Pic::Memory &>(vdata.memory()).normalizeWord(type, i, word);
   if ( type==Pic::MemoryRangeType::Config ) {
-    BitValue pmask = device().config()._words[i].pmask;
+    const BitValue pmask = device().config()._words[i].pmask;
     v = v.clearMaskBits(pmask);
     d = d.clearMaskBits(pmask);
   }
