@@ -411,7 +411,7 @@ virtual void processDevice(QDomElement device)
         case ProgVoltageType::Vpp:
         case ProgVoltageType::VddBulkErase: qFatal(QString("Voltage \"%1\" not defined").arg(vtype.key()));
         case ProgVoltageType::VddWrite: data()->_voltages[ProgVoltageType::VddWrite] = data()->_voltages[ProgVoltageType::VddBulkErase]; break;
-        case ProgVoltageType::Nb_Types: Q_ASSERT(false); break;
+        case ProgVoltageType::Nb_Types: ASSERT(false); break;
       }
     }
   }
@@ -504,7 +504,7 @@ virtual void processDevice(QDomElement device)
     if ( protection.family()==Protection::BasicProtection ) {
       QString maskName = protection.maskName(Protection::ProgramProtected, MemoryRangeType::Code);
       const Pic::Config::Mask *mask = data()->_config->findMask(maskName);
-      Q_ASSERT(mask);
+      ASSERT(mask);
       for (uint i=0; i<uint(mask->values.count()); i++) valueNames[mask->values[i].name] = false;
     }
     QDomNode child = checksums.firstChild();
@@ -513,7 +513,7 @@ virtual void processDevice(QDomElement device)
       if ( child.nodeName()!="checksum" ) qFatal("Childs of \"checksums\" should \"checksum\"");
       QString valueName = getChecksumData(child.toElement());
       if ( protection.family()==Protection::BasicProtection ) {
-        if ( !valueNames.contains(valueName) ) qFatal("Unknown protected attribute");
+        if ( !valueNames.contains(valueName) ) qFatal(QString("Unknown protected attribute in checksum \"%1\"").arg(valueName));
         valueNames[valueName] = true;
       }
       child = child.nextSibling();
@@ -543,7 +543,7 @@ void processMirrored(QDomElement element)
     mirrored.append(rd);
     child = child.nextSibling();
   }
-  if ( !mirrored.isEmpty() ) static_cast<RegistersData *>(data()->_registersData)->mirrored.append(mirrored);
+  if ( !mirrored.isEmpty() ) static_cast<RegistersData *>(data()->_registersData.get())->mirrored.append(mirrored);
 }
 
 void processUnused(QDomElement e)
@@ -555,7 +555,7 @@ void processUnused(QDomElement e)
   Address end = fromHexLabel(e.attribute("end"), &ok);
   rd.length = end-rd.start+1;
   if (!ok) qFatal("Malformed end for unused register");
-  static_cast<RegistersData *>(data()->_registersData)->unused.append(rd);
+  static_cast<RegistersData *>(data()->_registersData.get())->unused.append(rd);
 }
 
 void processSfr(QDomElement e)
@@ -589,7 +589,7 @@ void processSfr(QDomElement e)
     rdata.bits[k].por = RegisterBitState(fromHex(por[i].latin1(), &ok));
     if ( !ok || rdata.bits[k].por>Nb_RegisterBitStates ) qFatal(QString("Malformed por bit %1").arg(k));
   }
-  static_cast<RegistersData *>(data()->_registersData)->sfrs[name] = rdata;
+  static_cast<RegistersData *>(data()->_registersData.get())->sfrs[name] = rdata;
 }
 
 void processCombined(QDomElement e)
@@ -608,7 +608,7 @@ void processCombined(QDomElement e)
   if ( !ok || rdata.nbChars<2 ) qFatal(QString("Combined %1 size %2 is malformed").arg(name).arg(e.attribute("size")));
   Address end = rdata.address + rdata.nbChars/2 - 1;
   if ( end>=rlength ) qFatal(QString("Address %1 outside register range").arg(toHexLabel(end, 3)));
-  static_cast<RegistersData *>(data()->_registersData)->combined[name] = rdata;
+  static_cast<RegistersData *>(data()->_registersData.get())->combined[name] = rdata;
 }
 
 void processDeviceRegisters(QDomElement element)
@@ -617,12 +617,13 @@ void processDeviceRegisters(QDomElement element)
   if ( !s.isEmpty() ) {
     if ( !_map.contains(s) ) qFatal(QString("Registers same as unknown device %1").arg(s));
     const Pic::Data *d = static_cast<const Pic::Data *>(_map[s]);
-    data()->_registersData = d->_registersData;
+    const RegistersData& rd = *static_cast<const RegistersData*>(d->_registersData.get());
+    data()->_registersData.reset(new RegistersData(rd));
     return;
   }
 
-  RegistersData &rdata = *static_cast<RegistersData *>(data()->_registersData);
-  bool ok;
+  RegistersData &rdata = *static_cast<RegistersData*>(data()->_registersData.get());
+  bool ok = false;
   rdata.nbBanks = element.attribute("nb_banks").toUInt(&ok);
   if ( !ok || data()->registersData().nbBanks==0 ) qFatal("Malformed number of banks");
   if ( data()->is18Family() ) {
