@@ -85,7 +85,7 @@ public:
     QColorGroup ncg = cg;
     if (_device) {
       const Device::Data *device = Device::Lister::instance().data(text(0));
-      Q_ASSERT(device);
+      ASSERT(device);
       ncg.setColor(QColorGroup::Text, Device::statusColor(device->status()));
     }
     KListViewItem::paintCell(p, ncg, column, width, align);
@@ -107,7 +107,7 @@ DeviceChooser::Dialog::Dialog(const QString &device, QWidget *parent)
   : ::Dialog(parent, "device_chooser_dialog", true, i18n("Select a device"),
              Ok|Close, Close, false, QSize(400, 300))
 {
-  setButtonOK(KGuiItem(i18n( "&Select"), "button_ok"));
+  setButtonGuiItem(Ok, KGuiItem(i18n( "&Select"), "dialog-ok"));
   QVBoxLayout *top = new QVBoxLayout(mainWidget(), 0, 10);
 
   // view
@@ -116,7 +116,7 @@ DeviceChooser::Dialog::Dialog(const QString &device, QWidget *parent)
   _listTypeCombo.reset(new EnumComboBox<ListType>("list_type", mainWidget()));
   connect(_listTypeCombo->combo(), SIGNAL(activated(int)), SLOT(updateList()));
   vbox->addWidget(_listTypeCombo->combo());
-  QPushButton *button = new KPushButton(KGuiItem(i18n("Reset Filters"), "reload"), mainWidget());
+  QPushButton *button = new KPushButton(KGuiItem(i18n("Reset Filters"), "view-refresh"), mainWidget());
   connect(button, SIGNAL(clicked()), SLOT(resetFilters()));
   vbox->addWidget(button);
   vbox->addStretch(1);
@@ -176,7 +176,7 @@ DeviceChooser::Dialog::Dialog(const QString &device, QWidget *parent)
   QStringList names;
   for (uint i=0; Device::Package::TYPE_DATA[i].name; i++) names += Device::Package::TYPE_DATA[i].name;
   qSort(names);
-  for (uint i=0; i<names.count(); i++) _packageCombo->appendItem(names[i], i18n(Device::Package::data(names[i])->label));
+  for (int i=0; i<names.count(); i++) _packageCombo->appendItem(names[i], i18n(Device::Package::data(names[i])->label));
   _packageCombo->setCurrentItem(Config::package());
   connect(_packageCombo->widget(), SIGNAL(activated(int)), SLOT(updateList()));
   fcontainer->appendWidget(_packageCombo->widget());
@@ -299,8 +299,6 @@ const Tool::Group *DeviceChooser::Dialog::toolGroup() const
 
 void DeviceChooser::Dialog::updateList(const QString &device)
 {
-  QMap<QString, KListViewItem *> groups;
-  Q3ListViewItem *selected = NULL;
   const Programmer::Group *pgroup = programmerGroup();
   if ( pgroup && pgroup->supportedDevices().isEmpty() && pgroup->isSoftware() ) {
     _deviceView->setText(i18n("Could not detect supported devices for \"%1\". Please check installation.").arg(pgroup->label()));
@@ -311,13 +309,16 @@ void DeviceChooser::Dialog::updateList(const QString &device)
     _deviceView->setText(i18n("Could not detect supported devices for toolchain \"%1\". Please check installation.").arg(tgroup->label()));
     return;
   }
-  QStringList list = Device::Lister::instance().supportedDevices();
+
+  QMap<QString, KListViewItem *> groups;
+  Q3ListViewItem *selected = NULL;
+  const QStringList list = Device::Lister::instance().supportedDevices();
   QStringList::const_iterator it = list.end();
-  for (it--; it!=list.end(); it--) {
+  for (it--; it!=list.begin(); it--) {
     if ( pgroup && !pgroup->isSupported(*it) ) continue;
     if ( tgroup && !tgroup->isSupported(*it) ) continue;
     const Device::Data *data = Device::Lister::instance().data(*it);
-    Q_ASSERT(data);
+    ASSERT(data);
     if ( _memoryCombo->value()!=Device::MemoryTechnology::Nb_Types && data->memoryTechnology()!=_memoryCombo->value() ) continue;
     if ( _statusCombo->value()!=Device::Status::Nb_Types && data->status()!=_statusCombo->value() ) continue;
     if ( _featureCombo->value()!=Pic::Feature::Nb_Types ) {
@@ -348,7 +349,7 @@ void DeviceChooser::Dialog::updateList(const QString &device)
       case ListType::Flat:
         item = new ListItem(_listView, *it, true, true);
         break;
-      case ListType::Nb_Types: Q_ASSERT(false); break;
+      case ListType::Nb_Types: ASSERT(false); break;
     }
     if ( device==(*it) ) selected = item;
   }
@@ -362,8 +363,9 @@ void DeviceChooser::Dialog::updateList(const QString &device)
 
 //-----------------------------------------------------------------------------
 DeviceChooser::ComboBox::ComboBox(QWidget *parent)
-  : KComboBox(parent, "device_chooser_combo")
+  : KComboBox(parent)
 {
+  setObjectName("device_chooser_combobox");
   for (Device::Lister::ConstIterator it = Device::Lister::instance().begin();
        it != Device::Lister::instance().end();
        ++it) {
@@ -386,8 +388,7 @@ DeviceChooser::Button::Button(QWidget *parent)
   _combo = new ComboBox(this);
   connect(_combo, SIGNAL(activated(int)), SIGNAL(changed()));
   hbox->addWidget(_combo);
-  QPixmap icon = IconLoader::loadIcon("fileopen", IconLoader::Toolbar);
-  KPushButton *button = new KPushButton(icon, QString::null, this);
+  KPushButton *button = new KPushButton(KIcon("document-open"), QString::null, this);
   connect(button, SIGNAL(clicked()), SLOT(chooseDevice()));
   hbox->addWidget(button);
 }
@@ -402,15 +403,16 @@ void DeviceChooser::Button::chooseDevice()
 
 //-----------------------------------------------------------------------------
 DeviceChooser::Browser::Browser(QWidget *parent)
- : KTextBrowser(parent, "device_browser")
+ : KTextBrowser(parent),
+   _data(NULL)
 {}
 
 PURL::Url findDocumentUrl(const QString &prefix, const QString &baseName)
 {
-  PURL::Url previous = KURL::fromPathOrURL(prefix + baseName + ".pdf");
+  PURL::Url previous = KUrl(prefix + baseName + ".pdf");
   bool previousExists = previous.exists();
   for (uint i=0; i<26; i++) {
-    PURL::Url url = KURL::fromPathOrURL(prefix + baseName + QChar('a' + i) + ".pdf");
+    PURL::Url url = KUrl(prefix + baseName + QChar('a' + i) + ".pdf");
     bool exists = url.exists();
     if ( !exists && previousExists ) return previous;
     previous = url;
@@ -419,16 +421,29 @@ PURL::Url findDocumentUrl(const QString &prefix, const QString &baseName)
   return previous;
 }
 
-void DeviceChooser::Browser::setSource(const QString &name)
+void DeviceChooser::Browser::setSource(const QUrl& url)
 {
   BusyCursorStarter bc;
-  if ( name.startsWith("device://") ) emit deviceChanged(name.mid(9));
-  else if ( name.startsWith("document://") ) {
+  if (url.scheme() == "device") emit deviceChanged(url.toString(QUrl::RemoveScheme));
+  else if (url.scheme() == "document") {
     QString prefix = "http://ww1.microchip.com/downloads/en/DeviceDoc/";
-    PURL::Url url = findDocumentUrl(prefix, name.mid(11, name.length()-11-1));
-    KTextBrowser::setSource(url.kurl().htmlURL());
-  }
-  else KTextBrowser::setSource(name);
+    PURL::Url foundUrl = findDocumentUrl(prefix, url.toString(QUrl::RemoveScheme));
+    KTextBrowser::setSource(foundUrl.kurl());
+  } else KTextBrowser::setSource(url);
+}
+
+void DeviceChooser::Browser::setData(const Device::Data* data)
+{
+  _data = data;
+}
+
+QVariant DeviceChooser::Browser::loadResource(int type, const QUrl& url)
+{
+  if (type != QTextDocument::ImageResource) return KTextBrowser::loadResource(type, url);
+  QString label = url.path();
+  if (label.startsWith("vf_")) return voltageFrequencyPixmap(*_data, label.mid(3));
+  else if (label.startsWith("pins_")) return pinsPixmap(*_data);
+  else return _data->group().memoryGraph(*_data);
 }
 
 //-----------------------------------------------------------------------------
@@ -437,24 +452,20 @@ DeviceChooser::View::View(QWidget *parent)
 {
   // Information
   _info = new Browser(this);
-  _info->setMimeSourceFactory(&_msf);
   insertTab(_info, i18n("Information"));
   connect(_info, SIGNAL(deviceChanged(const QString &)),
           SIGNAL(deviceChanged(const QString &)));
 
   // Memory Map
   _memory = new Browser(this);
-  _memory->setMimeSourceFactory(&_msf);
   insertTab(_memory, i18n("Memory Map"));
 
   // Voltage-Frequency Graphs
   _vfg = new Browser(this);
-  _vfg->setMimeSourceFactory(&_msf);
   insertTab(_vfg, i18n("Voltage-Frequency Graphs"));
 
   // Pin Diagrams
   _pins = new Browser(this);
-  _pins->setMimeSourceFactory(&_msf);
   insertTab(_pins, i18n("Pin Diagrams"));
 }
 
@@ -465,14 +476,16 @@ void DeviceChooser::View::setDevice(const QString &name, bool cannotChangeDevice
   QString doc = htmlInfo(*data, (cannotChangeDevice ? "" : "device:%1"), Device::documentHtml(*data));
   doc += Device::supportedHtmlInfo(*data);
   _info->setText("<html><body>" + doc + "</body></html>");
-  doc = htmlVoltageFrequencyGraphs(*data, QString::null, &_msf);
-  QPixmap pix = data->group().memoryGraph(*data);
+  _info->setData(data);
+  doc = voltageFrequencyHtml(*data, "vf_");
   QString label = data->name() + "_memory_map.png";
-  _msf.setPixmap(label, pix);
   _memory->setText("<html><body><img src=\"" + label + "\" /></body></html>");
+  _memory->setData(data);
   _vfg->setText("<html><body>" + doc + "</body></html>");
-  doc = htmlPinDiagrams(*data, QString::null, &_msf);
+  _vfg->setData(data);
+  doc = pinsHtml(*data, "pins_");
   _pins->setText("<html><body>" + doc + "</body></html>");
+  _pins->setData(data);
 }
 
 void DeviceChooser::View::setText(const QString &text)
