@@ -16,21 +16,37 @@
 #include "common/gui/misc_gui.h"
 
 Log::Widget::Widget(QWidget *parent, const char *name)
-  : KTextEdit(parent, name), _selectedLine(0)
+  : KTextEdit(parent)
 {
+  setObjectName(name);
   setTextFormat(Qt::RichText);
-  setWordWrap(WidgetWidth);
-  setWrapPolicy(Anywhere);
+  setLineWrapMode(WidgetWidth);
+  setWordWrapMode(QTextOption::WrapAnywhere);
   setUndoRedoEnabled(false);
   QPalette p = palette();
   setReadOnly(true);
   setPalette(p);
   setMinimumWidth(300);
+
+  connect(this, SIGNAL(aboutToShowContextMenu(QMenu*)), this, SLOT(slotAboutToShowContextMenu(QMenu*)));
 }
 
-void Log::Widget::updateDebugLevel()
+void Log::Widget::slotAboutToShowContextMenu(QMenu* popup)
 {
   setDebugLevel(GlobalConfig::debugLevel());
+
+  popup->insertItem(KIcon("document-save"), i18n("Save As..."), this, SLOT(saveAs()));
+  popup->insertItem(KIcon("edit-clear"), i18n("Clear"), this, SLOT(clear()));
+  popup->insertSeparator();
+  FOR_EACH(DebugLevel, level) {
+    _id[level.type()] = popup->insertItem(level.label());
+    popup->setItemChecked(_id[level.type()], _debugLevel==level);
+  }
+  popup->insertSeparator();
+  int id = popup->insertItem(i18n("Output in console"), this, SLOT(toggleConsoleOutput()));
+  popup->setItemChecked(id, GlobalConfig::logOutputType()==GuiConsole);
+  connect(popup, SIGNAL(activated(int)), SLOT(toggleVisible(int)));
+
 }
 
 void Log::Widget::logExtra(const QString &text)
@@ -61,31 +77,7 @@ void Log::Widget::doLog(const QString &text, const QString &color, bool bold, Ac
   append(s);
   // ### DANGER
   if ( action==Immediate)
-#ifdef NO_KDE
     QCoreApplication::processEvents(QEventLoop::ExcludeUserInput);
-#else
-    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput);
-#endif
-}
-
-Q3PopupMenu *Log::Widget::createPopupMenu(const QPoint &pos)
-{
-  updateDebugLevel();
-  _popup = KTextEdit::createPopupMenu(pos);
-  QPixmap icon = IconLoader::loadIcon("filesave", IconLoader::Small);
-  _popup->insertItem(icon, "Save As...", this, SLOT(saveAs()));
-  icon = IconLoader::loadIcon("fileclose", IconLoader::Small);
-  _popup->insertItem(icon, "Clear", this, SLOT(clear()));
-  _popup->insertSeparator();
-  FOR_EACH(DebugLevel, level) {
-    _id[level.type()] = _popup->insertItem(level.label());
-    _popup->setItemChecked(_id[level.type()], _debugLevel==level);
-  }
-  _popup->insertSeparator();
-  int id = _popup->insertItem(i18n("Output in console"), this, SLOT(toggleConsoleOutput()));
-  _popup->setItemChecked(id, GlobalConfig::logOutputType()==GuiConsole);
-  connect(_popup, SIGNAL(activated(int)), SLOT(toggleVisible(int)));
-  return _popup;
 }
 
 void Log::Widget::toggleVisible(int id)
@@ -125,36 +117,22 @@ void Log::Widget::clear()
 
 void Log::Widget::saveAs()
 {
-  PURL::Url url = PURL::getSaveUrl(":save_log", "text/x-log", this, i18n("Save log to file"), PURL::AskOverwrite);
+  PURL::Url url = PURL::getSaveUrl("kfiledialog:///save_log", "text/x-log", this, i18n("Save log to file"), PURL::AskOverwrite);
   if ( url.isEmpty() ) return;
   url.write(_text, *this);
 }
 
-QColor calculateAlternateBackgroundColor(const QColor& base)
+void Log::Widget::mouseReleaseEvent(QMouseEvent *e)
 {
-    if (base == Qt::white)
-        return QColor(238,246,255);
-    else
-    {
-        int h, s, v;
-        base.hsv( &h, &s, &v );
-        if (v > 128)
-            return base.dark(106);
-        else if (base != Qt::black)
-            return base.light(110);
-
-        return QColor(32,32,32);
-    }
-}
-
-void Log::Widget::contentsMouseReleaseEvent(QMouseEvent *e)
-{
-  KTextEdit::contentsMouseReleaseEvent(e);
+  KTextEdit::mouseReleaseEvent(e);
   if ( e->button()!=Qt::LeftButton ) return;
-  QColor color = colorGroup().base();
-  setParagraphBackgroundColor(_selectedLine, color);
-  _selectedLine = paragraphAt(e->pos());
-  color = calculateAlternateBackgroundColor(color);
-  setParagraphBackgroundColor(_selectedLine, color);
-  lineClicked(_selectedLine);
+  QTextBlockFormat format = _cursor.blockFormat();
+  format.clearBackground();
+  _cursor.setBlockFormat(format);
+  _cursor = cursorForPosition(e->pos());
+  format = _cursor.blockFormat();
+  QBrush brush = palette().brush(QPalette::AlternateBase);
+  format.setBackground(brush);
+  _cursor.setBlockFormat(format);
+  lineClicked(_cursor.block().firstLineNumber());
 }
