@@ -10,6 +10,8 @@
 #include "project_manager.h"
 #include "project_manager.moc"
 
+#include <krecentfilesaction.h>
+
 #include "project.h"
 #include "project_wizard.h"
 #include "project_editor.h"
@@ -162,67 +164,70 @@ void ProjectManager::View::contextMenu(Q3ListViewItem *item, const QPoint &p, in
 
   if ( item->rtti()==LinkerScriptRtti || group==Group::LinkerScript ) {
     if ( Main::toolGroup().linkerScriptType()==PURL::Nb_FileTypes ) return;
-    pop.insertTitle(i18n("Linker Script"));
-    pop.insertItem("piklab_addfile", i18n("Set Custom..."));
-    if ( !_project->customLinkerScript().isEmpty() ) pop.insertItem("editdelete", i18n("Set Default"));
-    switch( pop.exec(p) ) {
-      case 1: {
-        PURL::Url url = PURL::getOpenUrl(":custom_linker_script", PURL::filter(Main::toolGroup().linkerScriptType()), this, i18n("Select Linker Script"));
-        if ( !url.isEmpty() ) _project->setCustomLinkerScript(url);
-        break;
+    pop.addTitle(i18n("Linker Script"));
+    QAction* addAction = pop.addAction(KIcon("document-import"), i18n("Set Custom..."));
+    QAction* removeAction = !_project->customLinkerScript().isEmpty()
+      ? pop.addAction(KIcon("edit-delete"), i18n("Set Default")) : NULL;
+    QAction* action = pop.exec(p);
+    if ( action==addAction ) {
+      PURL::Url url = PURL::getOpenUrl("kfiledilog:///custom_linker_script", PURL::filter(Main::toolGroup().linkerScriptType()), this, i18n("Select Linker Script"));
+      if ( !url.isEmpty() ) {
+        _project->setCustomLinkerScript(url);
+        _linkerScriptItem->init();
       }
-      case 2: _project->setCustomLinkerScript(PURL::Url()); break;
+    } else if ( removeAction!=NULL && action==removeAction ) {
+      _project->setCustomLinkerScript(PURL::Url());
+      _linkerScriptItem->init();
     }
-    _linkerScriptItem->init();
   } else if ( item->rtti()==RootRtti ) {
     if (_project.get() != NULL) {
-      pop.insertTitle(i18n("Project"));
-      pop.insertItem("configure", i18n("Options..."), &MainWindow::self(), SLOT(configureProject()));
-      pop.insertItem("find", i18n("Find Files..."),   &MainWindow::self(), SLOT(runKfind()));
+      pop.addTitle(i18n("Project"));
+      pop.insertItem("document-properties", i18n("Options..."), &MainWindow::self(), SLOT(configureProject()));
+      pop.insertItem("edit-find", i18n("Find Files..."),   &MainWindow::self(), SLOT(runKfind()));
       pop.insertSeparator();
-      pop.insertItem("piklab_compile", i18n("Build Project"), &MainWindow::self(), SLOT(buildProject()));
-      pop.insertItem("trashcan_empty", i18n("Clean Project"), &MainWindow::self(), SLOT(cleanBuild()));
+      pop.insertItem("run-build", i18n("Build Project"), &MainWindow::self(), SLOT(buildProject()));
+      pop.insertItem("run-build-clean", i18n("Clean Project"), &MainWindow::self(), SLOT(cleanBuild()));
       pop.insertSeparator();
-      pop.insertItem("filenew", i18n("New Source File..."), &MainWindow::self(), SLOT(newSourceFile()));
-      pop.insertItem("piklab_addfile", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
-      pop.insertItem("piklab_addfile", i18n("Add Object Files..."), this, SLOT(insertObjectFiles()));
-      if (Main::currentEditor() != NULL) pop.insertItem("piklab_addcurrentfile", i18n("Add Current File"), this, SLOT(insertCurrentFile()));
+      pop.insertItem("document-new", i18n("New Source File..."), &MainWindow::self(), SLOT(newSourceFile()));
+      pop.insertItem("document-import", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
+      pop.insertItem("document-import", i18n("Add Object Files..."), this, SLOT(insertObjectFiles()));
+      if (Main::currentEditor() != NULL) pop.insertItem("document-import", i18n("Add Current File"), this, SLOT(insertCurrentFile()));
       pop.exec(p);
     }
   } else if ( item->rtti()==FileRtti ) {
     FileItem *fi = static_cast<FileItem *>(item);
     if ( isExternalFile(fi->url()) ) return;
-    pop.insertTitle(item->text(0));
-    pop.insertItem("editdelete", i18n("Remove From Project"));
-    if ( pop.exec(p)==1 ) removeFile(fi->url());
+    pop.addTitle(item->text(0));
+    pop.insertItem("edit-delete", i18n("Remove From Project"));
+    if ( pop.exec(p)!=NULL ) removeFile(fi->url());
   } else if ( item->rtti()==HeaderRtti ) {
     switch (group.type()) {
       case Group::LinkerObject:
-        pop.insertTitle(i18n("Objects"));
-        pop.insertItem("piklab_addfile", i18n("Add Object Files..."), this, SLOT(insertObjectFiles()));
+        pop.addTitle(i18n("Objects"));
+        pop.insertItem("document-import", i18n("Add Object Files..."), this, SLOT(insertObjectFiles()));
         pop.exec(p);
         break;
       case Group::Source:
       case Group::Header:
-        pop.insertTitle(i18n("Sources"));
-        pop.insertItem("filenew", i18n("New File..."), &MainWindow::self(), SLOT(newSourceFile()));
-        pop.insertItem("piklab_addfile", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
+        pop.addTitle(i18n("Sources"));
+        pop.insertItem("document-new", i18n("New File..."), &MainWindow::self(), SLOT(newSourceFile()));
+        pop.insertItem("document-import", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
         pop.exec(p);
         break;
       case Group::Device:
-        pop.insertTitle(i18n("Device"));
-        pop.insertItem("piklab_chip", i18n("Select..."), &MainWindow::self(), SLOT(showDeviceInfo()));
+        pop.addTitle(i18n("Device"));
+        pop.insertItem("media-flash", i18n("Select..."), &MainWindow::self(), SLOT(showDeviceInfo()));
         pop.exec(p);
         break;
       case Group::Toolchain:
-        pop.insertTitle(i18n("Toolchain"));
-        pop.insertItem("configure", i18n("Options..."), &MainWindow::self(), SLOT(configureProjectToolchain()));
-        pop.insertItem("exec", i18n("Configure Toolchains..."), &MainWindow::self(), SLOT(configureToolchains()));
+        pop.addTitle(i18n("Toolchain"));
+        pop.insertItem("document-properties", i18n("Options..."), &MainWindow::self(), SLOT(configureProjectToolchain()));
+        pop.insertItem("system-run", i18n("Configure Toolchains..."), &MainWindow::self(), SLOT(configureToolchains()));
         pop.exec(p);
         break;
       case Group::Programmer:
-        pop.insertTitle(i18n("Programmer"));
-        pop.insertItem("piklab_config_programmer", i18n("Configure Programmers..."), &MainWindow::self(), SLOT(configureProgrammers()));
+        pop.addTitle(i18n("Programmer"));
+        pop.insertItem("audio-card", i18n("Configure Programmers..."), &MainWindow::self(), SLOT(configureProgrammers()));
         pop.exec(p);
         break;
       case Group::LinkerScript:
@@ -270,14 +275,14 @@ void ProjectManager::View::addExternalFiles()
 
 void ProjectManager::View::insertSourceFiles()
 {
-  PURL::UrlList list = PURL::getOpenUrls(":<sources>", PURL::sourceFilter(PURL::CompleteFilter), this, i18n("Select Source File"));
+  PURL::UrlList list = PURL::getOpenUrls("kfiledialog:///sources", PURL::sourceFilter(PURL::CompleteFilter), this, i18n("Select Source File"));
   if ( list.isEmpty() ) return;
   for (PURL::UrlList::const_iterator it=list.begin(); it!=list.end(); ++it) insertFile(*it);
 }
 
 void ProjectManager::View::insertObjectFiles()
 {
-  PURL::UrlList list = PURL::getOpenUrls(":<objects>", PURL::objectFilter(PURL::CompleteFilter), this, i18n("Select Object File"));
+  PURL::UrlList list = PURL::getOpenUrls("kfiledialog:///objects", PURL::objectFilter(PURL::CompleteFilter), this, i18n("Select Object File"));
   if ( list.isEmpty() ) return;
   for (PURL::UrlList::const_iterator it=list.begin(); it!=list.end(); ++it) insertFile(*it);
 }
@@ -291,8 +296,9 @@ void ProjectManager::View::insertFile(const PURL::Url &url)
   PURL::Url purl = url;
   MessageBox::Result copy = MessageBox::No;
   if ( !url.isInto(_project->directory()) ) {
-    copy = MessageBox::questionYesNoCancel(i18n("File \"%1\" is not inside the project directory. Do you want to copy the file to your project directory?").arg(url.pretty()),
-                                          i18n("Copy and Add"), i18n("Add only"));
+    KGuiItem copyAdd(i18n("Copy and Add"));
+    KGuiItem addOnly(i18n("Add only"));
+    copy = MessageBox::questionYesNoCancel(i18n("File \"%1\" is not inside the project directory. Do you want to copy the file to your project directory?").arg(url.pretty()), copyAdd, addOnly);
     if ( copy==MessageBox::Cancel ) return;
     if ( copy==MessageBox::Yes ) purl = PURL::Url(_project->directory(), url.filename());
   }
@@ -313,13 +319,13 @@ void ProjectManager::View::insertFile(const PURL::Url &url)
 
 bool ProjectManager::View::openProject()
 {
-  PURL::Url url = PURL::getOpenUrl(":open_project", PURL::projectFilter(PURL::CompleteFilter), this, i18n("Select Project file"));
+  PURL::Url url = PURL::getOpenUrl("kfiledialog:///open_project", PURL::projectFilter(PURL::CompleteFilter), this, i18n("Select Project file"));
   return openProject(url);
 }
 
 void ProjectManager::View::addExternalFile(const PURL::Url &url, FileOrigin origin)
 {
-  Q_ASSERT( origin!=Intrinsic );
+  ASSERT( origin!=Intrinsic );
   addFile(url, url.fileType(), origin);
 }
 
@@ -332,7 +338,6 @@ void ProjectManager::View::addFile(const PURL::Url &url, PURL::FileType type, Fi
     ext[url] = Included;
     return;
   }
-  PURL::FileProperties properties = type.data().properties;
   Group grp;
   switch (origin) {
     case Intrinsic: grp = group(type); break;
@@ -443,10 +448,12 @@ void ProjectManager::View::insertCurrentFile()
 
 bool ProjectManager::View::configureProject(ProjectEditor::PageType type)
 {
-  ProjectEditor dialog(*_project, type, this);
-  if ( dialog.exec()!=QDialog::Accepted ) return false;
-  _modified = true;
-  if (_linkerScriptItem) _linkerScriptItem->init();
+  if (_project.get() != NULL) {
+    ProjectEditor dialog(*_project, type, this);
+    if ( dialog.exec()!=QDialog::Accepted ) return false;
+    _modified = true;
+    if (_linkerScriptItem != NULL) _linkerScriptItem->init();
+  }
   return true;
 }
 
@@ -467,11 +474,12 @@ bool ProjectManager::View::newProject()
 bool ProjectManager::View::openProject(const PURL::Url& url)
 {
   if (url.isEmpty()) return false;
-  static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->removeURL(url.kurl());
+  static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->removeUrl(url.kurl());
   bool reload = (_project.get() != NULL && _project->url()==url);
+  KGuiItem reloadItem(i18n("Reload"));
   if (reload
-      && !MessageBox::askContinue(i18n("Project already loaded. Reload?"), i18n("Reload"))) {
-    static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->addURL(url.kurl());
+      && !MessageBox::askContinue(i18n("Project already loaded. Reload?"), reloadItem)) {
+    static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->addUrl(url.kurl());
     return false;
   }
   Project *p = new Project(url);
@@ -498,7 +506,7 @@ bool ProjectManager::View::openProject(const PURL::Url& url)
       addFile(*it, (*it).fileType(), Intrinsic);
   _projectData.type = PURL::Project;
   _projectData.externals.clear();
-  static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->addURL(url.kurl());
+  static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->addUrl(url.kurl());
   files = _project->openedFiles();
   for(PURL::UrlList::const_iterator it = files.begin(); it!=files.end(); ++it)
       Main::editorManager().openFile(*it);
