@@ -10,11 +10,7 @@
 #include "libgui/toplevel.h"
 #include "common/global/about.h"
 
-#ifndef NO_KDE
-#  include "libgui/likeback.h"
-#endif
-
-const KCmdLineOptions OPTIONS[] = {
+const Piklab::CmdLineOption OPTIONS[] = {
   { "+[file]", I18N_NOOP("Optional filenames to be opened upon startup."), NULL },
   { NULL, NULL, NULL}
 };
@@ -23,18 +19,11 @@ int main(int argc, char **argv)
 {
   Piklab::AboutData about("piklab", I18N_NOOP("Piklab"),
                           I18N_NOOP( "Graphical development environment for applications based on PIC and dsPIC microcontrollers."));
-  Piklab::initArgs(about, argc, argv, *OPTIONS);
+  Piklab::OptionList options(OPTIONS);
+  Piklab::initArgs(about, argc, argv, options);
   const bool gui = true;
   Piklab::createApplication(about.appName(), argc, argv, gui, NULL);
-#ifdef NO_KDE
-  MainWindow::self().show();
-  return QApplication::exec();
-#else
-  LikeBack::init(true);
-  LikeBack::setServer("piklab.sourceforge.net", "/likeback/send.php");
-  LikeBack::setAllowFeatureWishes(true);
-  if ( kapp->isRestored() && KMainWindow::canBeRestored(1) ) MainWindow::self().restore(1);
+  if ( kapp->isSessionRestored() && KMainWindow::canBeRestored(1) ) MainWindow::self().restore(1);
   else MainWindow::self().show();
   return kapp->exec();
-#endif
 }
