@@ -72,8 +72,7 @@ bool CustomProgrammer::ProcessManager::execute(const QString &command, const QSt
     _process->setWorkingDirectory(url.directory());
   }
   log(Log::LineType::Command, _process->arguments().join(" "));
-  Process::State state = Process::runSynchronouslyDialog(*_process, Process::Start,
-    i18n("Executing '%1' command...").arg(action));
+  Process::State state = Process::runSynchronouslyDialog(*_process, i18n("Executing '%1' command...").arg(action));
   if (state == Process::StartFailed) {
     log(Log::LineType::Error, i18n("*** Error executing command ***"));
     return false;
