@@ -36,10 +36,9 @@ bool GPSim::Process::runSynchronously()
 {
   uint retries = 0;
   for (;;) {
-    _state = ::Process::Running;
-    ::Process::State state = ::Process::runSynchronously(*this, ::Process::NoRunAction, 200);
+    execute(200);
     // this is because sometimes the synchronous process get stuck and timeout
-    if ( state!=::Process::Timedout || _ready ) return true;
+    if ( state()!=::Process::Timedout || _ready ) return true;
     ++retries;
     if ( retries == 20 ) break;
   }
@@ -61,7 +60,7 @@ void GPSim::Process::dataReceivedSlot(::Process::OutputType type)
           || _ready) {
       log(Log::DebugLevel::Extra, "received console prompt", Log::Delayed);
       _ready = true;
-      emit requestSynchronousStop();
+      _stopExecute = true;
     }
   }
 }
@@ -101,7 +100,7 @@ bool GPSim::ProcessManager::start(const PURL::Directory& execDir)
   exec += "gpsim";
   _process._ready = false;
   _process.setup(exec, QStringList("-i"), false);
-  if ( !_process.start(0) ) {
+  if ( !_process.start() ) {
     log(Log::LineType::Error, i18n("Failed to start \"gpsim\"."));
     return false;
   }
@@ -152,7 +151,7 @@ bool GPSim::ProcessManager::getVersion(VersionData &version)
   if (!sendCommand("version", true)) return false;
   version = VersionData();
   QRegExp reg("\\w*\\s*(\\d+\\.\\d+\\.\\d+).*");
-  for (uint i = 0; i<outputLines().count(); ++i) {
+  for (int i = 0; i<outputLines().count(); ++i) {
     if (reg.exactMatch(outputLines()[i]) ) {
       version = VersionData::fromString(reg.cap(1));
       break;
@@ -199,7 +198,7 @@ void GPSim::Hardware::internalDisconnectHardware()
 
 bool GPSim::Hardware::execute(const QString &command, bool synchronous, QStringList *output)
 {
-  Q_ASSERT(_manager.get() != NULL);
+  ASSERT(_manager.get() != NULL);
   log(Log::DebugLevel::Normal, QString("command: %1").arg(command));
   if (output) output->clear();
   if ( !_manager->sendCommand(command, synchronous) ) return false;
@@ -209,7 +208,7 @@ bool GPSim::Hardware::execute(const QString &command, bool synchronous, QStringL
 
 bool GPSim::Hardware::signal(uint n, bool synchronous, QStringList *output)
 {
-  Q_ASSERT(_manager.get() != NULL);
+  ASSERT(_manager.get() != NULL);
   log(Log::DebugLevel::Normal, QString("signal: %1").arg(n));
   if (output) output->clear();
   if ( !_manager->sendSignal(n, synchronous) ) return false;
