@@ -12,5 +12,5 @@
 
 QPixmap Debugger::OptionsConfigWidget::pixmap() const
 {
-  return IconLoader::loadIcon("piklab_config_debugger", IconLoader::Toolbar);
+  return IconLoader::loadIcon("tool-report-bug", IconLoader::Toolbar);
 }
