@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
+ *   Copyright (C) 2005-2010 Nicolas Hadacek <hadacek@kde.org>             *
  *   Copyright (C) 2002-2003 Stephen Landamore <stephen@landamore.com>     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -63,7 +63,7 @@ QString Icd2::TestData::result(VoltageType type) const
 {
   if ( _voltages[type]==-1 ) return "---";
   FOR_EACH(::Programmer::ResultType, rtype)
-    if ( _voltages[type]==RESULT_TYPE_VALUES::data(rtype) ) return type.label();
+    if ( _voltages[type]==RESULT_TYPE_VALUES::data(rtype) ) return rtype.label();
   return toHex(_voltages[type], 2);
 }
 
@@ -88,25 +88,25 @@ const Icd2::Hardware::WRITE_MODE_VALUES::Data Icd2::Hardware::WRITE_MODE_VALUES:
 const char *Icd2::Hardware::readCommand(Pic::MemoryRangeType type) const
 {
   switch (type.type()) {
-    case Pic::MemoryRangeType::Code:                          return "47";
-    case Pic::MemoryRangeType::Eeprom:                        return "48";
+    case Pic::MemoryRangeType::Code:                           return "47";
+    case Pic::MemoryRangeType::Eeprom:                         return "48";
     case Pic::MemoryRangeType::Config:
     case Pic::MemoryRangeType::CalBackup:
-      if ( device().architecture()==Pic::Architecture::P18J ) return "47";
-      else                                                    return "49";
+      if (device().architecture()==Pic::Architecture::P18J)    return "47";
+      else                                                     return "49";
     case Pic::MemoryRangeType::UserId:
-      if (device().architecture() == Pic::Architecture::P33F) return "49";
-      else                                                    return "4A";
+      if (device().architecture() == Pic::Architecture::P33F)  return "49";
+      else                                                     return "4A";
     case Pic::MemoryRangeType::DeviceId:
-      if ( device().is16bitFamily() )                         return "49";
-      else                                                    return "4A";
+      if ( device().is16bitFamily() )                          return "49";
+      else                                                     return "4A";
     case Pic::MemoryRangeType::Cal:
-      if (device().isCalibrationAtCodeEnd())                  return "47";
-      else                                                    return "49";
-    case Pic::MemoryRangeType::DebugVector:                   return "40";
+      if (device().isCalibrationAtCodeEnd())                   return "47";
+      else                                                     return "49";
+    case Pic::MemoryRangeType::DebugVector:                    return "40";
     case Pic::MemoryRangeType::ProgramExecutive:
     case Pic::MemoryRangeType::HardwareStack:
-    case Pic::MemoryRangeType::Nb_Types:                      break;
+    case Pic::MemoryRangeType::Nb_Types:                       break;
   }
   ASSERT(false);
   return NULL;
@@ -115,23 +115,24 @@ const char *Icd2::Hardware::readCommand(Pic::MemoryRangeType type) const
 const char *Icd2::Hardware::writeCommand(Pic::MemoryRangeType type) const
 {
   switch (type.type()) {
-    case Pic::MemoryRangeType::Code:                          return "43";
-    case Pic::MemoryRangeType::Eeprom:                        return "44";
+    case Pic::MemoryRangeType::Code:                           return "43";
+    case Pic::MemoryRangeType::Eeprom:                         return "44";
     case Pic::MemoryRangeType::Config:
     case Pic::MemoryRangeType::CalBackup:
-      if ( device().architecture()==Pic::Architecture::P18J ) return "43";
-      else                                                    return "45";
+      if (device().architecture()==Pic::Architecture::P18J
+	  || device().architecture()==Pic::Architecture::P24F) return "43";
+      else                                                     return "45";
     case Pic::MemoryRangeType::UserId:
-      if (device().architecture() == Pic::Architecture::P33F) return "45";
-      else                                                    return "46";
-    case Pic::MemoryRangeType::DeviceId:                      break;
+      if (device().architecture() == Pic::Architecture::P33F)  return "45";
+      else                                                     return "46";
+    case Pic::MemoryRangeType::DeviceId:                       break;
     case Pic::MemoryRangeType::Cal:
-      if (device().isCalibrationAtCodeEnd())                  return "43";
-      else                                                    return "45";
-    case Pic::MemoryRangeType::DebugVector:                   return "41";
+      if (device().isCalibrationAtCodeEnd())                   return "43";
+      else                                                     return "45";
+    case Pic::MemoryRangeType::DebugVector:                    return "41";
     case Pic::MemoryRangeType::ProgramExecutive:
     case Pic::MemoryRangeType::HardwareStack:
-    case Pic::MemoryRangeType::Nb_Types:                      break;
+    case Pic::MemoryRangeType::Nb_Types:                       break;
   }
   ASSERT(false);
   return NULL;
@@ -168,7 +169,8 @@ bool Icd2::Hardware::setTarget()
 bool Icd2::Hardware::setup()
 {
   // ??
-  if ( device().architecture()==Pic::Architecture::P30F )
+  if ( device().architecture()==Pic::Architecture::P30F
+      || device().architecture()==Pic::Architecture::P24F)
     if ( !command("0900", 0) ) return false;
 
   // ??
@@ -214,8 +216,9 @@ bool Icd2::Hardware::receiveResponse(const QString &command, uint responseSize,
     if ( !static_cast<USBPort *>(_port)->poll(size, _rx) ) return false;
   } else if ( !_port->receive(size, _rx, 180000) ) return false; // is 3 minutes enough ?? (we should really have an abort button here...)
   log(Log::DebugLevel::Extra, QString("received answer: '%1'").arg(_rx));
-  if ( size!=fromHex(_rx.mid(1, 2), 0) ) {
-    log(Log::LineType::Error, i18n("Received length too short."));
+  const uint receivedSize = fromHex(_rx.mid(1, 2), 0);
+  if ( size!=receivedSize) {
+    log(Log::LineType::Error, i18n("Received length mismatch: received %1, expected: %2.").arg(receivedSize).arg(size));
     return false;
   }
   if ( uint(_rx.length())!=size ) {
@@ -300,7 +303,7 @@ bool Icd2::Hardware::getTargetMode(Device::TargetMode &tmode)
     tmode = mode;
     return true;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return false;
 }
 
@@ -335,7 +338,7 @@ bool Icd2::Hardware::readBlock(uint nbBytesWord, uint nbWords, Device::Array &da
   }
 
   // treat data
-  log(Log::DebugLevel::Max, QString("received %1 * %2: %2").arg(nbBytesWord).arg(nbWords).arg(s));
+  log(Log::DebugLevel::Max, QString("received %1 * %2: %3").arg(nbBytesWord).arg(nbWords).arg(s));
   if ( s[0]!='{' || s[s.length()-1]!='}' ) {
     log(Log::LineType::Error, i18n("Invalid begin or end character for read block."));
     return false;
@@ -400,7 +403,7 @@ bool Icd2::Hardware::readMemory(Pic::MemoryRangeType type, uint wordOffset,
 bool Icd2::Hardware::writeBlock(uint nbBytesWord, const Device::Array &data, uint wordIndex, uint nbWords)
 {
   log(Log::DebugLevel::Extra, QString("writeBlock offset:%1 nbWords:%2 (size: %3)").arg(toHex(wordIndex, 8)).arg(toHex(nbWords, 8)).arg(toHex(data.size(), 8)));
-  Q_ASSERT( wordIndex+nbWords<=data.size() );
+  ASSERT( wordIndex+nbWords<=data.size() );
   // prepare data
   QString s = "{";
   uchar chk = 0;
@@ -469,7 +472,8 @@ bool Icd2::Hardware::eraseAll()
   if ( hasError() ) return false;
   if ( !sendCommand("29") ) return false;
   if ( !receiveResponse("29", 0, true) ) return false; // poll
-  if (device().architecture() == Pic::Architecture::P33F) {
+  if (device().architecture() == Pic::Architecture::P33F
+      || device().architecture() == Pic::Architecture::P24F) {
     if (!command("25", 0)) return false;
   }
   return true;
