@@ -46,7 +46,7 @@ bool Pickit2::USBPort::readFirmwareCodeMemory(Device::Array &data, const Device:
     if ( !receiveWords(1, 1, read) ) return false;
     for (uint k=0; k<nb; k++) {
       uint index = i + start + k;
-      data[index]= read[5 + 2*k] & 0xFF | (read[6 + 2*k] << 8);
+      data[index]= (read[5 + 2*k] & 0xFF) | (read[6 + 2*k] << 8);
       if ( vdata && index>=0x1000 && index<0x3FF0 && data[index]!=(*vdata)[index] ) {
         log(Log::LineType::Error, i18n("Firmware memory does not match hex file (at address 0x%2: reading 0x%3 and expecting 0x%4).")
                     .arg(QString(toHex(index/2, device.nbCharsAddress())))
@@ -299,7 +299,7 @@ bool Pickit2::P18F::doEraseCommand(uint cmd1, uint cmd2)
 bool Pickit2::P18F::doEraseRange(Pic::MemoryRangeType type)
 {
   if ( type==Pic::MemoryRangeType::Eeprom ) return doEraseCommand(0x84, 0x00);
-  Q_ASSERT( type==Pic::MemoryRangeType::Code );
+  ASSERT( type==Pic::MemoryRangeType::Code );
   if ( !doEraseCommand(0x81, 0x00) ) return false; // boot
   for (uint i=0; i<device().config().protection().nbBlocks(); i++)
     if ( !doEraseCommand(1 << i, 0x80) ) return false;
@@ -308,8 +308,8 @@ bool Pickit2::P18F::doEraseRange(Pic::MemoryRangeType type)
 
 bool Pickit2::P18F::doRead(Pic::MemoryRangeType type, uint wordOffset, Device::Array &data, const ::Programmer::VerifyData *vdata)
 {
-  Q_ASSERT( wordOffset==0 );
-  Q_ASSERT( data.count()==device().nbWords(type) );
+  ASSERT( wordOffset==0 );
+  ASSERT( data.count()==device().nbWords(type) );
   USBPort &port = static_cast<USBPort &>(hardware().port());
   QValueVector<uint> words;
   switch (type.type()) {
@@ -361,14 +361,14 @@ bool Pickit2::P18F::doRead(Pic::MemoryRangeType type, uint wordOffset, Device::A
     case Pic::MemoryRangeType::ProgramExecutive:
     case Pic::MemoryRangeType::Nb_Types: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return false;
 }
 
 bool Pickit2::P18F::doWrite(Pic::MemoryRangeType type, uint wordOffset, const Device::Array &data, bool force)
 {
-  Q_ASSERT( wordOffset==0 );
-  Q_ASSERT( data.count()==device().nbWords(type) );
+  ASSERT( wordOffset==0 );
+  ASSERT( data.count()==device().nbWords(type) );
   Q_UNUSED(force);
   USBPort &port = static_cast<USBPort &>(hardware().port());
   switch (type.type()) {
@@ -424,7 +424,7 @@ bool Pickit2::P18F::doWrite(Pic::MemoryRangeType type, uint wordOffset, const De
     case Pic::MemoryRangeType::ProgramExecutive:
     case Pic::MemoryRangeType::Nb_Types: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return false;
 }
 
@@ -439,7 +439,7 @@ bool Pickit2::P18F2X20::doEraseCommand(uint cmd1)
 bool Pickit2::P18F2X20::doEraseRange(Pic::MemoryRangeType type)
 {
   if ( type==Pic::MemoryRangeType::Eeprom ) return doEraseCommand(0x81);
-  Q_ASSERT( type==Pic::MemoryRangeType::Code );
+  ASSERT( type==Pic::MemoryRangeType::Code );
   if ( !doEraseCommand(0x83) ) return false; // boot
   for (uint i=0; i<device().config().protection().nbBlocks(); i++)
     if ( !doEraseCommand(0x88 + i) ) return false;
