@@ -25,7 +25,7 @@ VersionData Pickit2V2::Base::firmwareVersion(Programmer::FirmwareVersionType typ
     case Programmer::FirmwareVersionType::Max: return VersionData(2, 10, 0);
     case Programmer::FirmwareVersionType::Nb_Types: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return VersionData();
 }
 
