@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
+ *   Copyright (C) 2005-2010 Nicolas Hadacek <hadacek@kde.org>             *
  *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -89,17 +89,13 @@ void Compile::LogWidget::clear()
 void Compile::LogWidget::appendLine(Log::LineType type, const QString &message, const LogData &data)
 {
   log(type, message, Log::Delayed);
-  _map[paragraphs()-1] = data;
+  _map[document()->blockCount()-1] = data;
 }
 
 void Compile::LogWidget::lineClicked(int line)
 {
   if ( !_map.contains(line) ) return;
-#ifdef NO_KDE
   TextEditor *e = qobject_cast<TextEditor *>(Main::editorManager().openEditor(_map[line].url));
-#else
-  TextEditor *e = ::qt_cast<TextEditor *>(Main::editorManager().openEditor(_map[line].url));
-#endif
   if (e == NULL) return;
   e->setCursorPosition(_map[line].line, 0);
 }
@@ -220,7 +216,7 @@ bool Compile::BaseProcess::start()
   _process->setWorkingDirectory(directory().path());
   setupProcess();
   _manager->log(Log::LineType::Command, _process->arguments().join(" "));
-  return _process->start(0); // no timeout
+  return _process->start();
 }
 
 void Compile::BaseProcess::done(int code)
@@ -248,7 +244,7 @@ PURL::Url Compile::BaseProcess::url(PURL::FileType type, uint i) const
   else if (type==PURL::Library || _data.category==Tool::Category::Librarian)
     return Main::project()->url().toExtension(libraryExtension());
   else {
-    ASSERT( i<_data.items.count() );
+    ASSERT( i<(uint)_data.items.count() );
     url = _data.items[i].url;
   }
   if ( type==PURL::Nb_FileTypes ) return url;
@@ -272,7 +268,7 @@ Compile::FileData::List Compile::BaseProcess::files(bool *ok) const
   FileData::List list;
   QRegExp rexp("PURL::(.*)");
   QStringList files = QStringList::split(" ", outputFiles());
-  for (uint i=0; i<files.count(); i++) {
+  for (int i=0; i<files.count(); i++) {
     if ( rexp.exactMatch(files[i]) ) {
       PURL::FileType type = PURL::FileType::fromKey(rexp.cap(1));
       if ( type==PURL::Nb_FileTypes ) {
@@ -317,7 +313,7 @@ bool Compile::Process::check() const
 QStringList Compile::Process::replaceAllValue(const IdData &idata) const
 {
   if (idata.id == ArgType::Common) return BaseProcess::replaceAllValue(idata);
-  Q_ASSERT(idata.id == ArgType::Normal);
+  ASSERT(idata.id == ArgType::Normal);
   NormalArgType::Type type = NormalArgType::Type(idata.index);
   Tool::ExecutableType execType = Compile::Config::executableType(group());
   QStringList args;
@@ -333,7 +329,7 @@ QStringList Compile::Process::replaceAllValue(const IdData &idata) const
         args += tmp.relativeTo(directory(), execType.data().separator);
       }
       break;
-    default: Q_ASSERT(false); break;
+    default: ASSERT(false); break;
   }
   return args;
 }
@@ -341,7 +337,7 @@ QStringList Compile::Process::replaceAllValue(const IdData &idata) const
 QString Compile::Process::replaceValue(const IdData &idata) const
 {
   if (idata.id == ArgType::Common) return BaseProcess::replaceValue(idata);
-  Q_ASSERT(idata.id == ArgType::Normal);
+  ASSERT(idata.id == ArgType::Normal);
   NormalArgType::Type type = NormalArgType::Type(idata.index);
   Tool::ExecutableType execType = Compile::Config::executableType(group());
   switch(type) {
@@ -371,7 +367,7 @@ QString Compile::Process::replaceValue(const IdData &idata) const
       return filepath(PURL::Object);
     default: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return QString::null;
 }
 
@@ -379,14 +375,14 @@ QString Compile::Process::postReplaceValue(const IdData &idata,
                                            const QString &arg) const
 {
   if (idata.id == ArgType::Common) return BaseProcess::postReplaceValue(idata, arg);
-  Q_ASSERT(idata.id == ArgType::Normal);
+  ASSERT(idata.id == ArgType::Normal);
   return arg;
 }
 
 bool Compile::Process::replaceIfCondition(const IdData &idata) const
 {
   if (idata.id == ArgType::Common) return BaseProcess::replaceIfCondition(idata);
-  Q_ASSERT(idata.id == ArgType::Normal);
+  ASSERT(idata.id == ArgType::Normal);
   NormalArgType::Type type = NormalArgType::Type(idata.index);
   switch(type) {
     case NormalArgType::ReplaceIfCustomLinkerScript:
@@ -397,7 +393,7 @@ bool Compile::Process::replaceIfCondition(const IdData &idata) const
     }
     default: break;
   }
-  Q_ASSERT(false);
+  ASSERT(false);
   return false;
 }
 
@@ -459,7 +455,8 @@ void Compile::Process::doLog(const QString &type, const QString &message, const
 void Compile::Process::doLog(Log::LineType type, const QString &message, const QString &surl, uint line)
 {
   if ( surl.isEmpty() ) {
-    _manager->log(type, message);
+    // GUI update is delayed so as not to miss some output lines
+    _manager->log(type, message, Log::Delayed);
     return;
   }
   PURL::Url url = PURL::fromPathOrUrl(surl, true);
